<?php

namespace LWS\WOOREWARDS\PRO\Ui;

// don't call the file directly
if (!defined('ABSPATH')) exit();

require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointspoolfilter.php';

/** Create the backend menu and settings pages. */
class Admin
{
	const POOL_OPTION_PREFIX = 'lws-wr-pool-option-';

	public function __construct()
	{
		\LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsPoolFilter::install();

		\add_action('admin_enqueue_scripts', array($this, 'scripts'));
		\add_filter('lws_woorewards_ui_loyalty_tab_get', array($this, 'getLoyaltyPage'));
		\add_filter('lws_adminpanel_pages_' . LWS_WOOREWARDS_PAGE, array($this, 'managePages'));
		\add_filter('lws_woorewards_admin_pool_general_settings', array($this, 'poolGeneralSettings'), 15, 2); // priority to set after
		\add_filter('lws_woorewards_ui_userspoints_filters', array($this, 'userspointsFilters'));

		// grab woocommerce styles if any to appends them on stygen
		\add_filter('woocommerce_enqueue_styles', array($this, 'grabWCStyles'), 0);

		// Add specific support text for configuration help
		\add_filter('lws_adm_support_contents', array($this, 'addSupportText'), 10, 2);

		foreach (array('lws_woorewards_wc_achievements_endpoint_slug', 'lws_woorewards_wc_badges_endpoint_slug', 'lws_woorewards_wc_my_account_endpoint_slug') as $slugOption)
			\add_filter('pre_update_option_' . $slugOption, array($this, 'warnAboutEndpoint404'), 10, 3);

		\add_action('lws_woorewards_after_delete_all', array($this, 'deleteAllData'));
	}

	function warnAboutEndpoint404($value = true, $oldValue = false, $option = false)
	{
		if ($value != $oldValue)
		{
			\lws_admin_add_notice(
				'warnAboutEndpoint404',
				sprintf(
					'<p>%s</p><p>%s</p>',
					__("You just changed a slug in MyAccount tabs.", 'woorewards-pro'),
					__("Sometimes, when trying to go to the Loyalty and Rewards Tab, you will see a “404 – Page does not exist” error. This is a known WordPress permalink issue. To solve it, go to your WordPress administration and go to <strong>Settings → Permalinks</strong>. Once you’re there, scroll to the bottom of the page and click the <strong>Save Changes</strong> button. That will solve the problem.", 'woorewards-pro')
				),
				array('level' => 'info', 'dismissible' => true, 'forgettable' => true)
			);
		}
		return $value;
	}

	public function scripts($hook)
	{
		\wp_register_style('lws-wre-pro-poolssettings', LWS_WOOREWARDS_PRO_CSS . '/poolssettings.min.css', array(), LWS_WOOREWARDS_PRO_VERSION);
		$deps = array('jquery', 'lws-base64', 'lws-tools');
		\wp_register_script('lws-wre-pro-poolssettings', LWS_WOOREWARDS_PRO_JS . '/poolssettings.js', $deps, LWS_WOOREWARDS_PRO_VERSION, true);

		if (false !== strpos($hook, LWS_WOOREWARDS_PAGE))
		{
			\do_action('lws_adminpanel_enqueue_lac_scripts', array('select'));

			\wp_enqueue_script('lws-radio');

			$tab = isset($_GET['tab']) ? $_GET['tab'] : '';
			if (strpos($hook, 'loyalty') !== false)
			{
				foreach ($deps as $dep)
					\wp_enqueue_script($dep);
				\wp_enqueue_script('lws-wre-pro-poolssettings');
				\wp_enqueue_style('lws-wre-pro-poolssettings');
				\wp_enqueue_style('lws-wre-pro-style', LWS_WOOREWARDS_PRO_CSS . '/style.css', array(), LWS_WOOREWARDS_PRO_VERSION);
			}
			else if (false !== strpos($hook, 'settings') && strpos($tab, 'woocommerce') !== false)
			{
				if (\class_exists('\WC_Frontend_Scripts'))
				{
					\WC_Frontend_Scripts::get_styles();
					if (isset($this->wcStyles))
					{
						foreach ($this->wcStyles as $style => $detail)
						{
							\wp_enqueue_style($style, $detail['src'], $detail['deps'], $detail['version'], $detail['media'], $detail['has_rtl']);
						}
					}
				}
			}
			else
			{
				\wp_enqueue_style('lws-wre-userspointsfilters', LWS_WOOREWARDS_PRO_CSS . '/userspointsfilters.css', array(), LWS_WOOREWARDS_PRO_VERSION);
			}

			\wp_enqueue_style('lws-wre-pool-content-edit', LWS_WOOREWARDS_PRO_CSS . '/poolcontentedit.css', array(), LWS_WOOREWARDS_PRO_VERSION);
		}
	}

	function grabWCStyles($scripts)
	{
		if (!isset($this->wcStyles))
			$this->wcStyles = $scripts;
		return $scripts;
	}

	protected function getCurrentPage()
	{
		if (isset($_REQUEST['page']) && ($current = \sanitize_text_field($_REQUEST['page'])))
			return $current;
		if (isset($_REQUEST['option_page']) && ($current = \sanitize_text_field($_REQUEST['option_page'])))
			return $current;
		return false;
	}

	/** Reorganise pages from the free version to the pro version */
	function managePages($pages)
	{
		$this->standardPages = $pages;
		$proPages = array();
		if (isset($this->standardPages['wr_resume']))
		{
			$proPages['wr_resume'] = $this->standardPages['wr_resume'];
		}

		$proPages['wr_customers'] = $this->getCustomerPage();
		$proPages['wr_loyalty'] = $this->getLoyaltyPage();
		$proPages['wr_wizard'] = $this->getWizardPage();
		$proPages['wr_features'] = $this->getFeaturesPage();
		$proPages['wr_appearance'] = $this->getAppearancePage();
		$proPages['wr_system'] = $this->getSystemPage();
		return $proPages;
	}

	function getCustomerPage()
	{
		$customerPage = $this->standardPages['wr_customers'];

		$customerPage['description'] = array(
			'cast' => 'p',
			__("The customers page lets you track your customers loyalty activity and perform some actions :", 'woorewards-pro'),
			array(
				'tag' => 'ul',
				__("See customers points and history", 'woorewards-pro'),
				__("See customers owned coupons", 'woorewards-pro'),
				__("Add/Subtract Points", 'woorewards-pro'),
				__("Add/Remove Rewards", 'woorewards-pro'),
				__("Add/Remove Badges", 'woorewards-pro'),
				__("Filter by points, activity or inactivity periods", 'woorewards-pro'),
			),
		);
		return $customerPage;
	}

	function getLoyaltyPage($tab = false)
	{
		$description = array(
			array(
				'tag' => 'p',
				__("Points and Rewards systems is the core mechanism of WooRewards.", 'woorewards-pro'),
				__("You can create an infinity of points and rewards systems working together or apart.", 'woorewards-pro'),
				__("For each system, you can set up the following options :", 'woorewards-pro'),
			),
			array(
				'tag' => 'ul',
				array(
					__("How to earn points :", 'woorewards-pro'),
					__("Select how your customers will earn points. More than 20 Methods available", 'woorewards-pro'),
				),
				array(
					__("Rewards :", 'woorewards-pro'),
					__("Set the rewards that your customers will receive for their loyalty", 'woorewards-pro'),
				),
				__("System Type : Use Standard systems or Leveling systems", 'woorewards-pro'),
				__("Currency : Choose how points will be displayed to users", 'woorewards-pro'),
				__("Restricted access : Restrict access to your systems depending on the user role", 'woorewards-pro'),
				__("Points Expiration : Choose between 3 methods for points to expire ... or not", 'woorewards-pro'),
			),
		);

		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/pools.php';
		$tabId = 'wr_loyalty';
		$title = __("Points and Rewards", 'woorewards-pro');

		$links = array('' => array('poolfilter' => ''));
		$labels = array('' => _x("All", "Points and rewards system filter", 'woorewards-pro'));
		foreach (\LWS\WOOREWARDS\PRO\Ui\Editlists\Pools::statusList() as $k => $status)
		{
			$links[$k] = array('poolfilter' => $k);
			$labels[$k] = $status;
		}
		$filter = new \LWS\Adminpanel\EditList\FilterSimpleLinks($links, array(), false, $labels);

		$loyaltyPage = array(
			'title' => __("Points and Rewards", 'woorewards-pro'),
			'color' => '#526981',
			'image'		=> LWS_WOOREWARDS_IMG . '/r-loyalty-systems.png',
			'description' => $description,
			'rights'   => 'manage_rewards',
			'id'       => LWS_WOOREWARDS_PAGE . '.loyalty',
			'tabs'   => array(
				$tabId  => array(
					'title'  => $title,
					'id'     => $tabId,
					'hidden' => true,
					'tabs'    => array(
						array(
							'title'  => $title,
							'id'     => 'systems',
							'hidden' => true,
							'groups' => array(
								'systems' => array(
									'id'       => 'systems',
									'title'    => __("Points and Rewards Systems", 'woorewards-pro'),
									'icon'	   => 'lws-icon-present',
									'color'    => '#016087',
									'text'     => array(
										'join' => '<br/>',
										__("Points and Rewards Systems are WooReward's core. This is how your customers <b>earn points and get rewards</b>.", 'woorewards-pro'),
										__("When adding a new system, you'll have two options :", 'woorewards-pro'),
										array(
											'tag' => 'ul',
											__("<b>Standard System</b> : Customers earn points in various ways and can spend their points to unlock rewards.", 'woorewards-pro'),
											__("<b>Leveling System</b> : Customers earn points and unlock levels and rewards as they progress. <b>In a leveling system, customers never spend their points</b>.", 'woorewards-pro'),
										),
									),
									'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/advanced-mechanisms/points-and-rewards-systems/'),
									'editlist' => \lws_editlist(
										\LWS\WOOREWARDS\PRO\Ui\Editlists\Pools::SLUG,
										\LWS\WOOREWARDS\PRO\Ui\Editlists\Pools::ROW_ID,
										new \LWS\WOOREWARDS\PRO\Ui\Editlists\Pools(),
										\LWS\Adminpanel\EditList::DDA,
										$filter
									)
								)
							)
						)
					)
				)
			)
		);

		// build only the required pool page
		$pool = $this->guessCurrentPool($tabId);
		if ($pool)
		{
			$subtab = $pool->getTabId();
			$loyaltyPage['tabs'][$tabId]['tabs'][$subtab] = \apply_filters('lws_woorewards_ui_loyalty_edit_pool_tab', array(
				'title'  => $pool->getOption('display_title'),
				'id'     => $subtab,
				'hidden' => true,
				'groups' => $this->getLoyaltyGroups($pool),
				'delayedFunction' => function () use ($pool)
				{
					echo "<div style='width:50%;'>";
					\do_action('wpml_show_package_language_ui', $pool->getPackageWPML());
					echo "</div>";
				}
			), $pool);
		}

		return $loyaltyPage;
	}

	function getWizardPage()
	{
		$customerPage = $this->standardPages['wr_wizard'];
		$customerPage['description'] = array(
			array(
				'tag' => 'p',
				__("Wizards are made to help you get started or create some specific points and rewards systems more easily.", 'woorewards-pro'),
				__("Select a wizard, follow the setup steps and everything will be generated automatically according to your preferences.", 'woorewards-pro'),
				__("Here are the wizards available :", 'woorewards-pro'),
			),
			array(
				'tag' => 'ul',
				array(
					__("Standard System :", 'woorewards-pro'),
					__("Set up a standard system to let customers win coupons", 'woorewards-pro'),
				),
				array(
					__("Leveling System :", 'woorewards-pro'),
					__("This wizard will help you create a bronze/silver/gold system", 'woorewards-pro'),
				),
				array(
					__("Special Events :", 'woorewards-pro'),
					__("points and rewards systems for special occasions like Christmas", 'woorewards-pro'),
				),
				array(
					__("Double Points :", 'woorewards-pro'),
					__("Create an event where customers can earn twice the points", 'woorewards-pro'),
				),
				array(
					__("Sponsorship :", 'woorewards-pro'),
					__("Sponsors and sponsored are rewarded in this points and rewards system", 'woorewards-pro'),
				),
				array(
					__("Birthday or Anniversary ", 'woorewards-pro'),
					__("Send a special gift on customers birthday or registration anniversary", 'woorewards-pro'),
				),
			)
		);
		return $customerPage;
	}

	function getFeaturesPage()
	{
		$description = array(
			'cast' => 'p',
			__("Activate and set up different WooRewards features in this section : ", 'woorewards-pro'),
			array(
				'tag' => 'ul',
				array(
					__("General Features :", 'woorewards-pro'),
					__("Enable or disable a list of general features that you can decide to use or not", 'woorewards-pro'),
				),
				array(
					__("Sponsorship/Referral :", 'woorewards-pro'),
					__("Set up the sponsorship/referral options", 'woorewards-pro'),
				),
				array(
					__("Badges and Achievements :", 'woorewards-pro'),
					__("Create and manage user badges, achievements and badges rarity", 'woorewards-pro'),
				),
				array(
					__("API :", 'woorewards-pro'),
					__("Set up the API to connect WooRewards with a third party app", 'woorewards-pro'),
				),
			)
		);

		$featuresPage = array_merge(
			$this->standardPages['wr_features'],
			array(
				'description'	=> $description,
				'tabs'     => array(
					'features' => $this->getGeneralFeaturesTab(),
					'sponsorship' => $this->getSponsorshipTab(),
					'badges_achievements' => $this->getBadgesAchievementsTab(),
					'api' => $this->getAPITab(),
				)
			)
		);

		return $featuresPage;
	}

	function getAppearancePage()
	{
		$description = array(
			'cast' => 'p',
			__("Set the appearance of everything your customers will see on your website regarding points and rewards systems : ", 'woorewards-pro'),
			array(
				'tag' => 'ul',
				array(
					__("Widgets :", 'woorewards-pro'),
					__("Setup the widgets/shortcodes options and appearance", 'woorewards-pro'),
				),
				array(
					__("Shortcodes :", 'woorewards-pro'),
					__("List of all shortcodes available in WooRewards and how to use them", 'woorewards-pro'),
				),
				array(
					__("WooCommerce :", 'woorewards-pro'),
					__("Additional loyalty information displayed on WooCommerce pages", 'woorewards-pro'),
				),
				array(
					__("Emails :", 'woorewards-pro'),
					__("Customize emails sent to your customers", 'woorewards-pro'),
				),
				array(
					__("Popup :", 'woorewards-pro'),
					__("Setup the popup that your customers see when they unlock a reward", 'woorewards-pro'),
				),
			)
		);

		$appearancePage = array_merge(
			$this->standardPages['wr_appearance'],
			array(
				'description'	=> $description,
				'tabs'     => array(
					'widgets' 	=> $this->getWidgetsTab(),
					'shortcodes' => $this->getShortcodesTab(),
					'woocommerce' => $this->getWooCommerceTab(),
					'emails' 	=> $this->getEmailsTab(),
					'popup' 	=> $this->getPopupTab(),
					/** Will be added when CE is ready */
					//'templates' => $this->getTemplatesTab(),
				)
			)
		);
		return $appearancePage;
	}

	protected function getOrderStatusList()
	{
		if (\function_exists('\wc_get_order_statuses'))
		{
			$orderStatus = array();
			foreach (\wc_get_order_statuses() as $value => $label)
			{
				if (substr($value, 0, 3) == 'wc-')
					$value = substr($value, 3);
				$orderStatus[] = array('value' => $value, 'label' => $label);
			}
			return $orderStatus;
		}
		else
		{
			return array(
				array('value' => 'pending', 'label' => __("Pending payment", 'woorewards-lite')),
				array('value' => 'processing', 'label' => __("Processing", 'woorewards-lite')),
				array('value' => 'on-hold', 'label' => __("On hold", 'woorewards-lite')),
				array('value' => 'completed', 'label' => __("Completed", 'woorewards-lite')),
				array('value' => 'cancelled', 'label' => __("Cancelled", 'woorewards-lite')),
				array('value' => 'refunded', 'label' => __("Refunded", 'woorewards-lite')),
				array('value' => 'failed', 'label' => __("Failed", 'woorewards-lite')),
			);
		}
	}

	function getGeneralFeaturesTab()
	{
		$lite = array(
			'settings' => $this->standardPages['wr_features']['tabs']['wc_settings']['groups']['settings'],
		);
		if (isset($this->standardPages['wr_features']['tabs']['wc_settings']['groups']['pointsoncart']))
			$lite['pointsoncart'] = $this->standardPages['wr_features']['tabs']['wc_settings']['groups']['pointsoncart'];
		foreach ($lite as &$grp)
			$grp['class'] = 'half';

		$gfTab = array(
			'id'	=> 'general_features',
			'title'	=>  __("General Features", 'woorewards-pro'),
			'icon'	=> 'lws-icon-questionnaire',
			'vertnav' => true,
			'groups' => array_merge($lite, array(
				'a_features' => array(
					'id' 	=> 'advanced_features',
					'title' => __("Advanced Features", 'woorewards-pro'),
					'icon'	=> 'lws-icon-adv-settings',
					'class'	=> 'half',
					'text' 	=> __("The following options are only used in a handful of cases and should only be enabled if you're certain you want to use them", 'woorewards-pro'),
					'fields' => array(
						'enable_leaderboard' => array(
							'id' => 'lws_woorewards_enable_leaderboard',
							'title' => __("Enable the leaderboard shortcode.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'help'	=> __("If you enable this shortcode, a new option will be added to the user's My Account page.
The user will have the possibility to set if he/she accepts to appear on the leaderboard or not.", 'woorewards-pro'),
							)
						),
					)
				),
				'birthday' => array(
					'id' 	=> 'wc_birthday',
					'title' => __("Birthday Settings", 'woorewards-pro'),
					'icon'	=> 'lws-icon-birthday-cake',
					'class'	=> 'half',
					'text' 	=> __("If you decide to give points for customers birthdays, you should check some of the following options to give customers the opportunity to fill in their birthday date.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/points/birthday/'),
					'fields' => array(
						'birthday_checkout' => array(
							'id' => 'lws_woorewards_registration_birthday_field',
							'title' => __("Display a birthday field in the 'checkout' page when the user register at the same time.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
							)
						),
						'birthday_register' => array(
							'id' => 'lws_woorewards_myaccount_register_birthday_field',
							'title' => __("Display a birthday field in the 'my account register' page.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
							)
						),
						'birthday_detail' => array(
							'id' => 'lws_woorewards_myaccount_detail_birthday_field',
							'title' => __("Display a birthday field in the 'my account -> details' page.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
							)
						)
					)
				),
				'refund' => array(
					'id'    => 'refund',
					'icon'	=> 'lws-icon-refund',
					'title' => __("Remove points on order refunds", 'woorewards-pro'),
					'class'	=> 'half',
					'text'  => __("If an order is cancelled or refunded, you have the option to also remove points earned for that order. Select here the order status for which you want to remove loyalty points.", 'woorewards-pro'),
					'fields' => array(
						'refund' => array(
							'id'    => 'lws_woorewards_refund_on_status',
							'title' => __("Order status", 'woorewards-pro'),
							'type'  => 'lacchecklist',
							'extra' => array(
								'source' => $this->getOrderStatusList(),
							)
						),
					)
				),
			)),
		);
		return $gfTab;
	}

	function getWooCommerceTab()
	{
		$pointsOnCart = $this->standardPages['wr_appearance']['tabs']['sty_widgets']['groups']['pointsoncart'];

		$wcTab = array(
			'id'	=> 'wc_settings',
			'title'	=>  __("WooCommerce", 'woorewards-pro'),
			'icon'	=> 'lws-icon-cart-2',
			'vertnav' => true,
			'groups' => array(
				'myaccountlrview' => array(
					'id' => 'myaccountlarview',
					'icon'	=> 'lws-icon-users',
					'title' => __("My Account - Loyalty", 'woorewards-pro'),
					'text' => __("Show to the customer all loyalty and rewards information in a dedicated 'Loyalty and Rewards' Tab inside WooCommerce's My Account.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/my-account-loyalty/'),
					'fields' => array(
						'endpont_loyalty' => array(
							'id' => 'lws_woorewards_wc_my_account_endpont_loyalty',
							'title' => __("Display the Loyalty and Rewards tab.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'default' => 'on'
							),
						),
						array(
							'id' => 'lws_woorewards_wc_my_account_lar_label',
							'title' => __("Loyalty and Rewards Tab Title", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __('Loyalty and Rewards', 'woorewards-pro'),
								'wpml' => "WooRewards - My Account - Loyalty and Rewards Tab Title",
							)
						),
						array(
							'id' => 'lws_woorewards_wc_my_account_endpoint_slug',
							'title' => __("Endpoint Slug", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => 'lws_woorewards',
							)
						),
						array(
							'id'    => 'lws_woorewards_wc_my_account_lar_options',
							'title' => __("Elements to display", 'woorewards-pro'),
							'type'  => 'checkgrid', // radiogrid is specific to the wizard
							'extra' => array(
								'source' => array(
									array('value' => 'coupons', 'active' => 'yes', 'label' => __("Available Coupons", 'woorewards-pro')),
									array('value' => 'rewards', 'active' => 'yes', 'label' => __("Unlockable Rewards", 'woorewards-pro')),
									array('value' => 'systems', 'active' => 'yes', 'label' => __("Points and rewards systems Details", 'woorewards-pro')),
									array('value' => 'history', 'active' => 'yes', 'label' => __("Customer Points History", 'woorewards-pro')),
									array('value' => 'sponsoremail', 'active' => '', 'label' => __("Sponsorship Mailing", 'woorewards-pro')),
									array('value' => 'sponsorlink', 'active' => '', 'label' => __("Referral Link", 'woorewards-pro')),
								),
								'dragndrop' => 'yes',
								'help' => __("Select the elements you want to display on the loyalty and rewards tab.", 'woorewards-pro') . "<br/><strong>" .
									__("You can rearrange the elements in the order you want by using drag and drop.", 'woorewards-pro') . "</strong>",
							),
						),
						array(
							'id' => 'lws_woorewards_wc_my_account_systems_list',
							'title' => __("Points and rewards systems", 'woorewards-pro'),
							'type' => 'lacchecklist',
							'extra' => array(
								'ajax' => 'lws_woorewards_pool_list',
								'help' => __("Select the points and rewards systems you want to display", 'woorewards-pro'),
							)
						),
						'expanded_display' => array(
							'id' => 'lws_woorewards_wc_myaccount_expanded',
							'title' => __("Expanded display", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'help' => __("Disables the accordion feature on the endpoint and expands all sections", 'woorewards-pro'),
							),
						),
						'leveling_bar' => array(
							'id' => 'lws_woorewards_wc_myaccount_levelbars',
							'title' => __("Leveling Progress Bar", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'help' => __("Displays a 'Current Progress' bar for leveling systems", 'woorewards-pro'),
							),
						),
						array(
							'id' => 'lws_wre_myaccount_lar_view',
							'type' => 'themer',
							'extra' => array(
								'template' => 'wc_loyalty_and_rewards',
								'css' => LWS_WOOREWARDS_PRO_CSS . '/loyalty-and-rewards.css',
								'prefix' => '--wr-lar-'
							)
						)
					)
				),
				'myaccountbadgesview' => array(
					'id' => 'myaccountbadgesview',
					'icon'	=> 'lws-icon-cockade',
					'title' => __("My Account - Badges", 'woorewards-pro'),
					'text' => __("Show to the customer all badges he owns in a 'Badges' Tab inside WooCommerce's My Account.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/my-account-badges/'),
					'fields' => array(
						'endpoint_badges' => array(
							'id' => 'lws_woorewards_wc_my_account_endpoint_badges',
							'title' => __("Display the Badges tab.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'default' => 'on'
							),
						),
						array(
							'id' => 'lws_woorewards_wc_my_account_badges_label',
							'title' => __("Badges Tab Title", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __('My Badges', 'woorewards-pro'),
								'wpml' => "WooRewards - My Account - Badges Tab Title",
							)
						),
						array(
							'id' => 'lws_woorewards_wc_badges_endpoint_slug',
							'title' => __("Endpoint Slug", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => 'lws_badges',
							)
						),
						array(
							'id' => 'lws_wre_myaccount_badges_view',
							'type' => 'themer',
							'extra' => array(
								'template' => 'wc_badges_endpoint',
								'css' => LWS_WOOREWARDS_PRO_CSS . '/badges-endpoint.css',
								'prefix' => '--wr-badges-'
							)
						)
					)
				),
				'myaccountachievementsview' => array(
					'id' => 'myaccountachievementsview',
					'icon'	=> 'lws-icon-trophy',
					'title' => __("My Account - Achievements", 'woorewards-pro'),
					'text' => __("Show to the customer all possible achievements in a 'Achievements' Tab inside WooCommerce's My Account.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/my-account-achievements/'),
					'fields' => array(
						'endpoint_achievements' => array(
							'id' => 'lws_woorewards_wc_my_account_endpoint_achievements',
							'title' => __("Display the Achievements tab.", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'default' => 'on'
							),
						),
						array(
							'id' => 'lws_woorewards_wc_my_account_achievements_label',
							'title' => __("Achievements Tab Title", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __('Achievements', 'woorewards-pro'),
								'wpml' => "WooRewards - My Account - Achievements Tab Title",
							)
						),
						array(
							'id' => 'lws_woorewards_wc_achievements_endpoint_slug',
							'title' => __("Endpoint Slug", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => 'lws_achievements',
							)
						),
						array(
							'id' => 'lws_wre_myaccount_achievements_view',
							'type' => 'themer',
							'extra' => array(
								'template' => 'wc_achievements_endpoint',
								'css' => LWS_WOOREWARDS_PRO_CSS . '/achievements-endpoint.css',
								'prefix' => '--wr-achievements-'
							)
						)
					)
				),
				'pointsoncart' => $pointsOnCart,
				'cartcouponsview' => array(
					'id' => 'cartcouponsview',
					'icon'	=> 'lws-icon-coupon',
					'title' => __("Cart Coupons", 'woorewards-pro'),
					'text' => __("Show to the customer his available coupons. That block stay hidden if customer doesn't have coupons.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/available-coupons/'),
					'fields' => array(
						array(
							'id'    => 'lws_woorewards_apply_coupon_by_reload',
							'title' => __("Reload page to apply coupon", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'tooltips' => __("Using a custom cart widget can prevent the default javascript behavior. In that case, check that option to force a page reload when customer apply a coupon.", 'woorewards-pro'),
							),
						),
						array(
							'id' => 'lws_woorewards_cart_collaterals_coupons', // legacy id: coupon view position
							'title' => __("Location", 'woorewards-pro'),
							'type' => 'select',
							'extra' => array(
								'maxwidth' => '500px',
								'default' => 'not_displayed',
								'notnull' => true,
								'options' => array(
									'not_displayed' => __("Not displayed at all", 'woorewards-pro'),
									'middle_of_cart' => __("Between products and totals", 'woorewards-pro'),
									'cart_collaterals' => __("Left of cart totals", 'woorewards-pro'),
									'on' => __("Bottom of the cart page", 'woorewards-pro'),
								),
							)
						),
						array(
							'id' => 'lws_wre_cart_coupons_view',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'cartcouponsview',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/cartcouponsview.css',
								'subids' => array(
									'lws_woorewards_title_cart_coupons_view' => "WooRewards - Coupons - Title",
									'lws_woorewards_cart_coupons_button' => "WooRewards - Coupons - Button",
								)
							)
						),
					)
				),
				'cartpointspreview' => array(
					'id' => 'cartpointspreview',
					'icon'	=> 'lws-icon-cart-2',
					'title' => __("Cart Page Preview", 'woorewards-pro'),
					'text' => __("Show points that a customer could earn with his current cart. That block stay hidden if customer does not earn points.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/cart-earned-points/'),
					'fields' => array(
						array(
							'id' => 'lws_woorewards_cart_potential_position',
							'title' => __("Location", 'woorewards-pro'),
							'type' => 'select',
							'extra' => array(
								'default' => 'not_displayed',
								'notnull' => true,
								'options' => array(
									'not_displayed' => __("Not displayed at all", 'woorewards-pro'),
									'middle_of_cart' => __("Between products and totals", 'woorewards-pro'),
									'cart_collaterals' => __("Left of cart totals", 'woorewards-pro'),
									'bottom_of_cart' => __("Bottom of the cart page", 'woorewards-pro'),
								)
							)
						),
						array(
							'id' => 'lws_woorewards_cpp_show_detail',
							'title' => __("Show Detail", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'tooltips' => __("Check this option if you want to show the methods to earn points detail", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_cpp_unlogged_text',
							'title' => __("Text for unlogged customers", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'wpml' => "WooRewards - Cart Points Preview - Unlogged Text",
								'tooltips' => __("Fill this if you want to show a text for unlogged customers", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_cpp_show_unlogged',
							'title' => __("Show for unlogged customers", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'tooltips' => __("Check this option if you want to show potentially earned points to unlogged customers", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_cart_potential_pool',
							'title' => __("Points and rewards systems", 'woorewards-pro'),
							'type' => 'lacchecklist',
							'extra' => array(
								'ajax' => 'lws_woorewards_pool_list',
								'tooltips' => __("If you select several systems, they will be displayed separately, one after the other.", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_wre_cart_points_preview',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'cartpointspreview',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/cartpointspreview.css',
								'subids' => array(
									'lws_woorewards_title_cpp' => "WooRewards - Cart Point Preview - Title",
								)
							)
						)
					)
				),
				'productpointspreview' => array(
					'id' => 'productpointspreview',
					'icon'	=> 'lws-icon-barcode',
					'title' => __("Product Page Preview", 'woorewards-pro'),
					'text' => __("Show points that a customer could earn purchasing a given product. That block stay hidden if the product produces no points.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/product-earned-points/'),
					'fields' => array(
						array(
							'id' => 'lws_woorewards_product_potential_position',
							'title' => __("Location", 'woorewards-pro'),
							'type' => 'select',
							'extra' => array(
								'default' => 'not_displayed',
								'notnull' => true,
								'options' => array(
									'not_displayed' => __("Not displayed at all", 'woorewards-pro'),
									'before_summary' => __("Before product summary", 'woorewards-pro'),
									'inside_summary' => __("Inside product summary", 'woorewards-pro'),
									'after_summary' => __("After product summary", 'woorewards-pro'),
								)
							)
						),
						array(
							'id' => 'lws_woorewards_ppp_unlogged_text',
							'title' => __("Text for unlogged customers", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'wpml' => "WooRewards - Product Points Preview - Unlogged Text",
								'tooltips' => __("Fill this if you want to show a text for unlogged customers", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_ppp_show_unlogged',
							'title' => __("Show for unlogged customers", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'default' => 'on',
								'class' => 'lws_checkbox',
								'tooltips' => __("Check this option if you want to show potentially earned points to unlogged customers", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_product_potential_pool',
							'title' => __("Points and rewards systems", 'woorewards-pro'),
							'type' => 'lacchecklist',
							'extra' => array(
								'ajax' => 'lws_woorewards_pool_list',
								'tooltips' => __("If you select several systems, they will be displayed separately, one after the other.", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_wre_product_points_preview',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'productpointspreview',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/productpointspreview.css',
								'subids' => array(
									'lws_woorewards_label_ppp' => "WooRewards - Product Points Preview - Title",
								)
							)
						)
					)
				),
				'neworderemail' => array(
					'id' => 'neworderemail',
					'icon'	=> 'lws-icon-letter',
					'title' => __("New Order Email Message", 'woorewards-pro'),
					'text' => __("Set a message for customers when they place a new order", 'woorewards-pro') . "<br/>"
						. __("Use the following options to display specific information on the email :", 'woorewards-pro')
						. "<ul><li><strong>[wr_wc_order_points] : </strong>" . __("displays all information about points earned in the current points and rewards system.", 'woorewards-pro') . "</li>"
						. "<li><strong>[order_points] : </strong>" . __("displays the points earned for this order in the current points and rewards systems.", 'woorewards-pro') . "</li>"
						. "<li><strong>[points_name] : </strong>" . __("displays the name of the points in the current points and rewards systems.", 'woorewards-pro') . "</li>"
						. "<li><strong>[system_name] : </strong>" . __("displays the title of the current points and rewards systems.", 'woorewards-pro') . "</li></ul>"
						. "<strong>" . __("If multiple points and rewards systems gave points with the order, the text will be repeated for each system.", 'woorewards-pro') . "</strong>",
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/woocommerce-integration/new-order-email/'),
					'fields' => array(
						array(
							'id' => 'lws_woorewards_wc_new_order_enable',
							'title' => __("Enable Email Message", 'woorewards-pro'),
							'type' => 'box',
							'extra' => array(
								'default' => 'off',
								'class' => 'lws_checkbox',
								'tooltips' => __("Check this option if you want to show a message in new order emails", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_wc_new_order_pools',
							'title' => __("Points and rewards systems", 'woorewards-pro'),
							'type' => 'lacchecklist',
							'extra' => array(
								'ajax' => 'lws_woorewards_pool_list',
								'tooltips' => __("If you select several systems, they will be displayed separately, one after the other when using the shortcode.", 'woorewards-pro'),
							)
						),
						array(
							'id' => 'lws_woorewards_wc_new_order_content',
							'type' => 'wpeditor',
							'title' => __("Email text", 'woorewards-pro'),
							'extra' => array(
								'editor_height' => 30,
								'default' => __("With this order, you will earn [wr_wc_order_points]", 'woorewards-pro'),
								'wpml'    => "WooRewards - New Order Email Message - Earning Points",
							)
						),
					)
				)
			),
		);
		return $wcTab;
	}

	function getSponsorshipTab()
	{
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/sponsoredreward.php';
		$spTab = array(
			'id'	=> 'sp_settings',
			'title'	=>  __("Sponsorship/Referral", 'woorewards-pro'),
			'icon'	=> 'lws-icon-handshake',
			'vertnav' => true,
			'groups' => array(
				'sponsorship' => array(
					'id' 	=> 'sponsorship',
					'icon'	=> 'lws-icon-handshake',
					'class'	=> 'half',
					'title'	=> __("Sponsorship/Referral Features", 'woorewards-pro'),
					'text' 	=> __("Here, you'll find the different tools customers can user to sponsor/refer their friends and the reward given to sponsored users.", 'woorewards-pro') .
						__("To reward the sponsors, either use the dedicated wizard or select an appropriate earning method inside a points and rewards system.", 'woorewards-pro'),
					'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards/sponsorship/'),
					'fields' => array(
						'enable' => array(
							'id'    => 'lws_woorewards_event_enabled_sponsorship',
							'title' => __("Enable Sponsorship", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
								'default' => 'on'
							)
						),
						'enableReferral' => array(
							'id'    => 'lws_woorewards_referral_back_give_sponsorship',
							'type'  => 'box',
							'title' => __("Allow sponsorship via referral link", 'woorewards-pro'),
							'extra' => array(
								'default' => 'on',
								'class' => 'lws_checkbox',
								'help' => __("When a visitor comes from a referral link and registers, he will be sponsored by the user that posted the link.", 'woorewards-pro')
							)
						),
						'enableSocial' => array(
							'id'    => 'lws_woorewards_socialshare_back_give_sponsorship',
							'type'  => 'box',
							'title' => __("Allow sponsorship via social share", 'woorewards-pro'),
							'extra' => array(
								'default' => '',
								'class' => 'lws_checkbox',
								'help' => __("When a visitor comes from a link posted on a social network and registers, he will be sponsored by the user that shared the post.", 'woorewards-pro')
							)
						),
						'max'    => array(
							'id' => 'lws_wooreward_max_sponsorship_count',
							'title' => __("Max sponsorships per customer", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'pattern' => '\d+',
								'default' => '0',
								'help' => __("Set the maximum sponsorships allowed for users. No restriction on empty value or zero (0).", 'woorewards-pro')
							)
						),
					)
				),
				'sp_reward' => array(
					'id' => 'sponsored',
					'icon'	=> 'lws-icon-present',
					'class'	=> 'half',
					'title' => __("Sponsored Reward", 'woorewards-pro'),
					'text' => __("Define the reward granted to the sponsored customers. <strong>Works only for emailing sponsorship.</strong>", 'woorewards-pro')
						. sprintf(__("If you want to grant a reward for customers sponsored with other methods, please read the %s.", 'woorewards-pro'), '<a href="https://plugins.longwatchstudio.com/docs/woorewards/sponsorship/#reward" target="_blank">' . __("documentation", 'woorewards-pro') . '</a>'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards/sponsorship/#reward'),
					'editlist' => \lws_editlist(
						'Sponsored',
						\LWS\WOOREWARDS\PRO\Ui\Editlists\SponsoredReward::ROW_ID,
						new \LWS\WOOREWARDS\PRO\Ui\Editlists\SponsoredReward(),
						\LWS\Adminpanel\EditList::MDA
					)->setPageDisplay(false)->setCssClass('sponsoredreward')->setRepeatHead(false),
					'function' => function ()
					{
						\wp_enqueue_script('lws-wre-pro-sponsoredreward', LWS_WOOREWARDS_PRO_JS . '/sponsoredreward.js', array('lws-adminpanel-editlist'), LWS_WOOREWARDS_PRO_VERSION, true);
						\wp_enqueue_style('lws-wre-pro-sponsoredreward', LWS_WOOREWARDS_PRO_CSS . '/sponsoredreward.min.css', array('lws-admin-controls'), LWS_WOOREWARDS_PRO_VERSION);
					}
				),
				'sp_mail_widget' => array(
					'id' => 'sponsor_widget_style',
					'icon'	=> 'lws-icon-users-mm',
					'title' => __("Sponsorship Widget", 'woorewards-pro'),
					'text' => __("In this Widget, customers can sponsor their friends.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards/sponsorship/#spwidget'),
					'fields' => array(
						'unconnected'    => array(
							'id' => 'lws_wooreward_sponsorship_nouser',
							'title' => __("Text displayed if user not connected", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __("Please log in if you want to sponsor your friends", 'woorewards-pro')
							)
						),
						'success'    => array(
							'id' => 'lws_wooreward_sponsorship_success',
							'title' => __("Text displayed on success", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __("A mail has been sent to your friend about us.", 'woorewards-pro')
							)
						),
						'allow' => array(
							'id'    => 'lws_woorewards_sponsorship_allow_unlogged',
							'title' => __("Allow unlogged users to sponsor", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => false,
								'class' => 'lws_checkbox',
								'tooltips' => __("If you enable the following feature, unlogged users will have to enter their email address in addition to sponsored addresses.", 'woorewards-pro')
							)
						),
						'redirect' => array(
							'id'    => 'lws_woorewards_sponsorhip_user_notfound',
							'title' => __("Redirection if user not found", 'woorewards-pro'),
							'type'  => 'lacselect',
							'extra' => array(
								'predefined' => 'page',
								'tooltips' => __("If an unlogged user tries to sponsor some friends, the system will try to find the appropriate user.", 'woorewards-pro')
									. '<br/>' . __("If no user is found, the user will be redirected to the page specified here, inviting them to register. If nothing is specified, they will stay on the same page", 'woorewards-pro')
							)
						),
						array(
							'id' => 'lws_woorewards_sponsor_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'wr_sponsorship',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/sponsor.css',
								'subids' => array(
									'lws_woorewards_sponsor_widget_title' => "WooRewards - Sponsor Widget - Title",
									'lws_woorewards_sponsor_widget_submit' => "WooRewards - Sponsor Widget - Button",
									'lws_woorewards_sponsor_widget_placeholder' => "WooRewards - Sponsor Widget - Placeholder",
								)
							)
						)
					)
				),
				'sp_referral_widget' => array(
					'id' => 'referral',
					'icon'	=> 'lws-icon-url',
					'title' => __("Referral Link", 'woorewards-pro'),
					'text' => __("In this Widget, customers get a referral link they can share.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards/sponsorship/#splink'),
					'fields' => array(
						'display' => array(
							'id'    => 'lws_woorewards_sponsorship_link_display',
							'title' => __("Default Display", 'woorewards-pro'),
							'type'  => 'lacselect',
							'extra' => array(
								'mode' => 'select',
								'source' => array(
									array('value' => 'link',	'label' => __('Url Link', 'woorewards-pro')),
									array('value' => 'qrcode',	'label' => __('QR Code', 'woorewards-pro')),
									array('value' => 'both',	'label' => __('Both', 'woorewards-pro')),
								),
							)
						),
						'page' => array(
							'id'    => 'lws_woorewards_sponsorship_link_page',
							'title' => __("Destination Page", 'woorewards-pro'),
							'type'  => 'lacselect',
							'extra' => array(
								'help' => __("Select the default destination of the referral link. If left empty, it will redirect to the same page it's placed", 'woorewards-pro'),
								'predefined' => 'page',
							)
						),
						array(
							'id' => 'lws_woorewards_referral_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'wr_referral',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/referral.css',
								'subids' => array(
									'lws_woorewards_referral_widget_message' => "WooRewards - Referral Widget - Header",
								)
							)
						)
					)
				)
			)
		);
		return $spTab;
	}

	function getBadgesAchievementsTab()
	{
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/rarity.php';
		$filters = array();
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/achievements.php';
		$editlist = \lws_editlist(
			\LWS\WOOREWARDS\PRO\Ui\Editlists\Achievements::SLUG,
			\LWS\WOOREWARDS\PRO\Ui\Editlists\Achievements::ROW_ID,
			new \LWS\WOOREWARDS\PRO\Ui\Editlists\Achievements(),
			\LWS\Adminpanel\EditList::MDA,
			$filters
		);

		$baTab = array(
			'id'	=> 'ba_settings',
			'title'	=>  __("Badges & Achievements", 'woorewards-pro'),
			'icon'	=> 'lws-icon-trophy',
			'vertnav' => true,
			'groups' => array(
				'ba_features' => array(
					'id' 	=> 'ba_features',
					'icon'	=> 'lws-icon-settings-gear',
					'class'	=> 'half',
					'title'	=> __("Badges & Achievements Features", 'woorewards-pro'),
					'text' 	=> __("Enable badges and achievements to add all their specific features to WooRewards.", 'woorewards-pro') .
						__("Achievements can be used to earn badges. Badges can be used to unlock rewards.", 'woorewards-pro'),
					'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/other-features/badges/'),
					'fields' => array(
						'badge-enable' => array(
							'id'    => 'lws_woorewards_manage_badge_enable',
							'title' => __("Enable Badge & Achievements", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => 'on',
								'class' => 'lws_checkbox',
								'help' => __("Enable/disable badge menu, rewards, earning methods and achievements system.", 'woorewards-pro')
							)
						)
					)
				),
				'badges' => array(
					'id' 	=> 'badges',
					'icon'	=> 'lws-icon-cockade',
					'class'	=> 'half',
					'title'	=> __("Badges Management", 'woorewards-pro'),
					'text' 	=> __("Badges are a special kind of wordpress posts. For that reason, they're handled in their own interface.", 'woorewards-pro') .
						__("Please follow the link below to add, edit or delete badges.", 'woorewards-pro'),
					'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/other-features/badges/'),
					'fields' => array(
						'badge-link' => array(
							'id'    => 'lws_woorewards_badge_link',
							'title' => __("Badges Management", 'woorewards-pro'),
							'type'  => 'custom',
							'extra' => array(
								'gizmo'   => true,
								'content' => sprintf("<a href='%s' target='_blank'>%s</a>", \esc_attr(\admin_url('edit.php?post_type=lws_badge')), __("Manage your badges", 'woorewards-pro')),
								'help'    => __("You can see and edit the badges by clicking the link down below.", 'woorewards-pro'),
							)
						)
					)
				),
				'achievements' => array(
					'id'    => 'achievements',
					'title' => __("Achievements", 'woorewards-pro'),
					'icon'	=> 'lws-icon-trophy',
					'editlist' => $editlist,
					'text' => __("After creating some badges, you can set up achievements that customers need to achieve in order to earn the badges.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/other-features/achievements/'),
					'function' => function ()
					{
						\wp_enqueue_style('lws-wr-achievements', LWS_WOOREWARDS_PRO_CSS . '/achievements.css', array(), LWS_WOOREWARDS_PRO_VERSION);
					}
				),
				'badge' => array(
					'id' => 'badge',
					'icon'	=> 'lws-icon-window-add',
					'title' => __("Badge Popup", 'woorewards-pro'),
					'text' => __("Style the popup that will be displayed to customers when they earn a new badge.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/other-features/badges/#2-%E2%80%93-badges-styling'),
					'fields' => array(
						'stygen' => array(
							'id' => 'lws_wr_badge_style',
							'title' => '',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'action',
								'template' => 'lws_wr_badge_style',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/badge.css',
							)
						)
					)
				),
				'rarity' => array(
					'id' => 'lws_woorewards_rarity_levels',
					'icon'	=> 'lws-icon-g-chart',
					'title' => __("Badges Rarity Levels", 'woorewards-pro'),
					'text' => __("Define the rarity levels of Badges.", 'woorewards-pro') . "<br/>" .
						__("The percentage value is the max percentage of users owning the badge to get the corresponding rarity.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/other-features/badges/#3-%E2%80%93-badges-rarity'),
					'editlist' => \lws_editlist(
						'Rarity',
						\LWS\WOOREWARDS\PRO\Ui\Editlists\BadgeRarity::ROW_ID,
						new \LWS\WOOREWARDS\PRO\Ui\Editlists\BadgeRarity(),
						\LWS\Adminpanel\EditList::MDA
					)->setPageDisplay(false)->setCssClass('lws-rarity-editlist')->setRepeatHead(false),
					'function' => function ()
					{
						\wp_enqueue_style('lws-wre-pro-srarity', LWS_WOOREWARDS_PRO_CSS . '/rarity.css', array('lws-admin-controls'), LWS_WOOREWARDS_PRO_VERSION);
					}

				)

			)
		);
		return $baTab;
	}

	function getAPITab()
	{
		$restPrefix = \trailingslashit(\get_rest_url()) . \LWS\WOOREWARDS\PRO\Core\Rest::getNamespace();
		$restPrefix = "<span class='lws-group-descr-copy lws_ui_value_copy'><div class='lws-group-descr-copy-text content' tabindex='0'>{$restPrefix}</div><div class='lws-group-descr-copy-icon lws-icon lws-icon-copy copy'></div></span>";
		$apiTab = array(
			'id'	=> 'api_settings',
			'title'	=>  __("API", 'woorewards-pro'),
			'icon'	=> 'lws-icon-api',
			'groups' => array(
				'api' => array(
					'id'     => 'api',
					'icon'	=> 'lws-icon-api',
					'title'  => __("REST API", 'woorewards-pro'),
					'text'   => sprintf(__("Define WooRewards REST API settings. API endpoint will be %s", 'woorewards-pro'), $restPrefix),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/api/'),
					'fields' => array(
						'enabled' => array(
							'id'    => 'lws_woorewards_rest_api_enabled',
							'title' => __("Enable REST API", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'class' => 'lws_checkbox',
							)
						),
						'wc_auth' => array(
							'id'    => 'lws_woorewards_rest_api_wc_auth',
							'title' => __("Allows authentification by WooCommerce REST API", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => 'on',
								'class' => 'lws_checkbox',
							)
						),
					)
				),
				'users' => array(
					'id'     => 'users',
					'icon'	=> 'lws-icon-users-mm',
					'title'  => __("User Permissions", 'woorewards-pro'),
					'text'   => __("Define the website users that can access the different features of the API", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/api/'),
					'fields' => array(
						'info' => array(
							'id'    => 'lws_woorewards_rest_api_user_info',
							'title' => __("Users allowed to read general information", 'woorewards-pro'),
							'type'  => 'lacchecklist',
							'extra' => array(
								'predefined' => 'user',
								'tooltips' => __("The checked users can get points and rewards system list.", 'woorewards-pro'),
							)
						),
						'read' => array(
							'id'    => 'lws_woorewards_rest_api_user_read',
							'title' => __("Users allowed to read user information", 'woorewards-pro'),
							'type'  => 'lacchecklist',
							'extra' => array(
								'predefined' => 'user',
								'tooltips' => __("The checked users can get other users point amounts and history.", 'woorewards-pro'),
							)
						),
						'write' => array(
							'id'    => 'lws_woorewards_rest_api_user_write',
							'title' => __("Users allowed to change user information", 'woorewards-pro'),
							'type'  => 'lacchecklist',
							'extra' => array(
								'predefined' => 'user',
								'tooltips' => __("The checked users can add points to other users.", 'woorewards-pro'),
							)
						),
					)
				),
			)
		);

		return $apiTab;
	}

	function getWidgetsTab()
	{
		$showPoints = $this->standardPages['wr_appearance']['tabs']['sty_widgets']['groups']['showpoints'];
		$widgetsTab = array(
			'id' 	=> 'sty_widgets',
			'title'	=>  __("Widgets", 'woorewards-pro'),
			'icon'	=> 'lws-icon-components',
			'vertnav' => true,
			'groups' => array(
				'showpoints' => array(
					'id'	=> $showPoints['id'],
					'title'	=> $showPoints['title'],
					'icon'	=> 'lws-icon-star-empty',
					'text'	=> $showPoints['text'],
					'extra'	=> $showPoints['extra'],
					'fields' => array(
						'spunconnected' => $showPoints['fields']['spunconnected'],
						'showpoints' => $showPoints['fields']['showpoints'],
					)
				),
				'coupons' => array(
					'id'	=> 'coupons',
					'title'	=> __('Owned Coupons', 'woorewards-pro'),
					'icon'	=> 'lws-icon-coupon',
					'text'	=> __("In this Widget, customers can see the WooCommerce coupons they own.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/owned-coupons/'),
					'fields' => array(
						'clunconnected'    => array(
							'id' => 'lws_wooreward_wc_coupons_nouser',
							'title' => __("Text displayed if user not connected", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __("Please log in to see the coupons you have", 'woorewards-pro'),
							)
						),
						'ifemptycoupon' => array(
							'id' => 'lws_wooreward_wc_coupons_empty',
							'title' => __("Text displayed if no coupon available", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __("No coupon available", 'woorewards-pro'),
							)
						),
						'couponslist' => array(
							'id' => 'lws_woorewards_wc_coupons_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'wc_shop_coupon',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/coupons.css',
								'subids' => array('lws_woorewards_wc_coupons_template_head' => "WooRewards - Coupons Widget - Header")
							)
						),
					)
				),
				'stdrewards' => array(
					'id'	=> 'stdrewards',
					'title'	=> __('Standard System Rewards', 'woorewards-pro'),
					'icon'	=> 'lws-icon-present',
					'text'	=> __("In this Widget, customers can see the Rewards they can unlock in a Standard points and rewards system.", 'woorewards-pro') . "<br/>"
						. sprintf(__("If you change the 'Reward Cost' text, use %s to display the reward cost (eg : 100 Points)", 'woorewards-pro'), "<span style='font-weight:bold;color:#366'>[rw_cost]</span>") . "<br/>"
						. sprintf(__("If you change the 'Need More Points' text, use %s to display the reward cost and %s to display the points still needed", 'woorewards-pro'), "<span style='font-weight:bold;color:#366'>[rw_cost]</span>", "<span style='font-weight:bold;color:#366'>[rw_more]</span>"),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/rewards/'),
					'fields' => array(
						'stdusegrid' => array(
							'id'    => 'lws_woorewards_rewards_use_grid',
							'title' => __("Use grid display instead of table display", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => 'on',
								'class' => 'lws_checkbox',
								'help' => __("Until WooRewards 3.4, this widget used html tables to display rewards.", 'woorewards-pro') . "<br/>"
									. __("If you've set up the widget before that version, checking that box will force you to style it again", 'woorewards-pro')
							)
						),
						'stdrewards' => array(
							'id' => 'lws_woorewards_rewards_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'rewards_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/' . (empty(\get_option('lws_woorewards_rewards_use_grid', 'on')) ? 'rewards.css' : 'gridrewards.css'),
								'subids' => array(
									'lws_woorewards_rewards_widget_unlock' => "WooRewards - Rewards Widget - Unlock Button",
									'lws_woorewards_rewards_widget_locked' => "WooRewards - Rewards Widget - Locked Button",
									'lws_woorewards_rewards_widget_cost' => "WooRewards - Rewards Widget - Reward Cost",
									'lws_woorewards_rewards_widget_more' => "WooRewards - Rewards Widget - More Points Needed",
								),
							)
						),
					)
				),
				'levrewards' => array(
					'id'	=> 'levrewards',
					'title'	=> __('Leveling System Rewards', 'woorewards-pro'),
					'icon'	=> 'lws-icon-g-chart',
					'text'	=> __("In this Widget, customers can see the Rewards they can win in a Levelling points and rewards system.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/rewards/'),
					'fields' => array(
						'levrewards' => array(
							'id' => 'lws_woorewards_loyalties_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'loyalties_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/loyalties.css',
								'help' => __("In this Widget, customers can see the Rewards they can win in a Levelling points and rewards system.", 'woorewards-pro') . "<br/>"
							)
						),
					)
				),
				'events' => array(
					'id'	=> 'events',
					'title'	=> __('Earning Points', 'woorewards-pro'),
					'icon'	=> 'lws-icon-trend-up',
					'text'	=> __("In this Widget, customers can see what they need to do in order to earn points", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/points/'),
					'fields' => array(
						'events' => array(
							'id' => 'lws_woorewards_events_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'events_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/events.css',
								'subids' => array(
									'lws_woorewards_events_widget_message' => "WooRewards - Earning methods - Header",
									'lws_woorewards_events_widget_text' => "WooRewards - Earning methods - Description",
								),
							)
						),
					)
				),
				'badges' => array(
					'id' => 'badges',
					'icon'	=> 'lws-icon-cockade',
					'title' => __("Badges", 'woorewards-pro'),
					'text' => __("In this Widget, customers can see the badges available and the ones they own", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/badges/'),
					'fields' => array(
						'badges' => array(
							'id' => 'lws_woorewards_badges_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'badges_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/badges.css',
								'subids' => array(
									'lws_woorewards_badges_widget_message' => "WooRewards - Badges - Title",
								),
							)
						),
					)
				),
				'achievements' => array(
					'id' => 'achievements',
					'icon'	=> 'lws-icon-trophy',
					'title' => __("Achievements", 'woorewards-pro'),
					'text' => __("In this Widget, customers can see the achievements and their progress", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/achievements/'),
					'fields' => array(
						'achievements' => array(
							'id' => 'lws_woorewards_achievements_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'achievements_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/achievements.css',
								'subids' => array(
									'lws_woorewards_achievements_widget_message' => "WooRewards - Achievements - Title",
								),
							)
						),
					)
				),
				'social' => array(
					'id' => 'social_share',
					'icon'	=> 'lws-icon-network-communication',
					'title' => __("Social Media", 'woorewards-pro'),
					'text' => __("With this Widget, customers can share a page link on social media.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/widgets/social-share/'),
					'fields' => array(
						array(
							'id' => 'lws_woorewards_smshare_socialmedias',
							'title' => __("Social Media", 'woorewards-pro'),
							'type' => 'lacchecklist',
							'extra' => array(
								'source'  => \LWS\WOOREWARDS\PRO\Core\Socials::instance()->asDataSource(),
								'default' => \LWS\WOOREWARDS\PRO\Core\Socials::instance()->getSupportedNetworks(),
								'help' => __("Select the social medias for which you want share buttons. Save your settings to see them appear on the styling tool", 'woorewards-pro'),
							)
						),
						'smdispunc' => array(
							'id'    => 'lws_woorewards_social_display_unconnected',
							'title' => __("Display Buttons if users not connected", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => '',
								'class' => 'lws_checkbox',
							)
						),
						'smdisconnected'    => array(
							'id' => 'lws_woorewards_social_text_unconnected',
							'title' => __("Text displayed if user not connected", 'woorewards-pro'),
							'type' => 'text',
							'extra' => array(
								'size' => '50',
								'placeholder' => __("Only logged in users can earn points for sharing", 'woorewards-pro'),
								'wpml' => "WooRewards - Social Share - Not connected",
							)
						),
						'smpopup' => array(
							'id'    => 'lws_woorewards_social_share_popup',
							'title' => __("Open share dialog as popup", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => '',
								'class' => 'lws_checkbox',
								'tooltips' => __("Default behavior opens share dialog in a new tab.", 'woorewards-pro')
							)
						),
						array(
							'id' => 'lws_woorewards_social_share_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'wr_social_share',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/social-share.css',
								'subids' => array(
									'lws_woorewards_social_share_widget_message' => "WooRewards - Social Share - Title",
									'lws_woorewards_social_share_widget_text' => "WooRewards - Social Share - Description",
								)
							)
						)
					)
				)
			)
		);
		return $widgetsTab;
	}

	function getShortcodesTab()
	{
		$shortcodesTab = array(
			'id' 	=> 'sty_shortcodes',
			'title'	=>  __("Shortcodes", 'woorewards-pro'),
			'icon'	=> 'lws-icon-shortcode',
			'vertnav' => true,
			'groups' => array(
				'shortcodes' => array(
					'id'	=> 'shortcodes',
					'title'	=> __('Shortcodes', 'woorewards-pro'),
					'icon'	=> 'lws-icon-shortcode',
					'text'	=> __("In this section, you will find various shortcodes you can use on your website. These shortcodes don't have a widget counterpart.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/shortcodes/shortcodes/'),
					'fields' => array(
						'simplepoints'    => array(
							'id' => 'lws_woorewards_sc_simple_points',
							'title' => __("Simple Points Display", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_simple_points system="system_name"]',
								'description' =>  __("This simple shortcode is used to display the points for a specific points and rewards system with no decoration.", 'woorewards-pro') . "<br/>" .
									__("This is very convenient if you want to display points within a phrase for example.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
								),
							)
						),
						'username'    => array(
							'id' => 'lws_woorewards_sc_username',
							'title' => __("User Name and Title", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_user_name title="yes" raw="no"]',
								'description' =>  __("This shortcode displays the user name and title", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'title',
										'desc' => __("Shows the title if user unlocked a title reward.", 'woorewards-pro'),
									),
									array(
										'option' => 'raw',
										'desc' => __("Defines if the name and title are put in stylable elements or not.", 'woorewards-pro'),
									),
								),
							)
						),
						'nextlevel'    => array(
							'id' => 'lws_woorewards_sc_next_level',
							'title' => __("Points to next level", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_next_level_points system="system_name"]',
								'description' =>  __("This shortcode displays the points needed to reach the next level/reward of a points and rewards system.", 'woorewards-pro') . "<br/>" .
									__("Use it to motivate customers to reach a higher level.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'prefix',
										'desc' => __("(Optional) The text displayed before the points needed.", 'woorewards-pro'),
									),
									array(
										'option' => 'suffix',
										'desc' => __("(Optional) The text displayed after the points needed.", 'woorewards-pro'),
									),
									array(
										'option' => 'currency',
										'desc' => __("(Optional)  If set, the points will be displayed with the points and rewards system's currency.", 'woorewards-pro'),
									),
								),
							)
						),
						'pointsoncart' => array(
							'id' => 'wr_points_on_cart',
							'title' => __("Points on Cart Tool", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => "[wr_points_on_cart system='system_name']",
								'description' =>  __("This shortcode is used to display the Points on Cart tool.", 'woorewards-pro') . "<br/>" .
									__("You can customize its appearance in the WooCommerce Tab.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
								),
							)
						),
						'maxwpointsoncart' => array(
							'id' => 'wr_max_points_on_cart',
							'title' => __("Maximum Point Amount on Cart", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => "[wr_max_points_on_cart system='system_name' raw='']",
								'description' =>  __("This shortcode will show the maximum quantity of Point that could be used on this cart.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'raw',
										'desc' => __("(Optional) If set, the amount will be a simple text. Otherwise, it will be presented inside a stylable element", 'woorewards-pro'),
									),
								),
							)
						),
						'userlevel'    => array(
							'id' => 'lws_woorewards_sc_userlevel',
							'title' => __("User Level", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_user_level system="system_name" nolevel="No level message"]',
								'description' =>  __("This shortcode displays the current user level in a leveling points and rewards system.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'nolevel',
										'desc' => __("The text displayed if the user didn't reach the first level", 'woorewards-pro'),
									),
								),
							)
						),
						'progressbar'    => array(
							'id' => 'lws_woorewards_sc_progressbar',
							'title' => __("Progress Bar", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_progress_bar system="system_name" header="Your Current Progress"]',
								'description' =>  __("This shortcode displays a progress bar for a points and rewards system.", 'woorewards-pro') . "<br/>" .
									__("This is very useful to incentivise customers to reach a higher level.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the progress bar", 'woorewards-pro'),
									),
								),
							)
						),
						'progressstyle' => array(
							'id' => 'lws_woorewards_progressbar_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'progressbar_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/progressbar.css',
							)
						),
						'history'    => array(
							'id' => 'lws_woorewards_sc_history',
							'title' => __("Points History", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => "[wr_show_history system='system_name1,system_name2' count='15']",
								'description' =>  __("This shortcode displays a user's points history for one or several points and rewards systems.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("(Optional) The points and rewards systems you want to display (comma separated). You can find this value in <strong>WooRewards → points and rewards systems</strong>, in the <b>Shortcode Attribute</b> column.", 'woorewards-pro'),
									),
									array(
										'option' => 'count',
										'desc' => __("(Optional) The number of rows displayed. Default is 15.", 'woorewards-pro'),
									),
								),
							)
						),
						'historystyle' => array(
							'id' => 'lws_woorewards_history_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'history_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_CSS . '/templates/history.css',
							)
						),
					),
				),
				'widgets' => array(
					'id'	=> 'widgets',
					'title'	=> __('Widgets Shortcodes', 'woorewards-pro'),
					'icon'	=> 'lws-icon-components',
					'text'	=> __("In this section, you will find shortcodes that also exist as widgets.", 'woorewards-pro') . "<br/>"
						. __("In some cases, shortcodes provide extra options that aren't available in widgets.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/shortcodes/widget-shortcodes/'),
					'fields' => array(
						'showpoints'    => array(
							'id' => 'lws_woorewards_sc_show_points',
							'title' => __("Display Points", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_show_points system="system_name" force="true" title="your title" more_details_url="more details button url"]',
								'description' =>  __("This shortcode shows to customers the points they have on a points and rewards system.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'force',
										'desc' => __("(Optional) If set, the points will be shown even if the user currently doesn’t have access to the points and rewards system.", 'woorewards-pro'),
									),
									array(
										'option' => 'title',
										'desc' => __("(Optional) The text displayed before the points.", 'woorewards-pro'),
									),
									array(
										'option' => 'more_details_url',
										'desc' => __("(Optional) An url linking to a page with more details on the points and rewards systems.", 'woorewards-pro'),
									),
									array(
										'option' => 'show_currency',
										'desc' => __("(Optional) If set, the number of points displayed will show the points currency.", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_showpoints',
							)
						),
						'coupons'    => array(
							'id' => 'lws_woorewards_sc_shop_coupons',
							'title' => __("Owned Coupons", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_shop_coupons header=”Here is a list of your coupons”]',
								'description' =>  __("This shortcode shows to customers the woocommerce coupons they currently have.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the coupons.", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_coupons',
							)
						),
						'showrewards'    => array(
							'id' => 'lws_woorewards_sc_show_rewards',
							'title' => __("Rewards", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_show_rewards system="system_name" shared="true" force="true" title="your title" granted="all|only|excluded"]',
								'description' =>  __("This shortcode shows to customers the rewards they can earn in a points and rewards system.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'title',
										'desc' => __("The text displayed before the rewards.", 'woorewards-pro'),
									),
									array(
										'option' => 'shared',
										'desc' => __("If systems share the same points pool, you can show the rewards of all shared systems together. <strong>Warning</strong> : This only works if systems use the same type (Standard or Leveling).", 'woorewards-pro'),
									),
									array(
										'option' => 'force',
										'desc' => __("If set, the points will be shown even if the user currently doesn’t have access to the points and rewards system.", 'woorewards-pro'),
									),
									array(
										'option' => 'granted',
										'desc' => __("Select the rewards you want to show to customers : ", 'woorewards-pro')
											. "<ul><li><strong>all</strong> : " . __("All rewards of the selected points and rewards system", 'woorewards-pro') . "</li>"
											. "<li><strong>only</strong> : " . __("Only for logged users – Show rewards they can unlock with their points", 'woorewards-pro') . "</li>"
											. "<li><strong>excluded</strong> : " . __("Only for logged users – Show rewards for which users don’t have enough points", 'woorewards-pro') . "</li></ul>",
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_stdrewards',
							)
						),
						'earning'    => array(
							'id' => 'lws_woorewards_sc_earning_points',
							'title' => __("Actions to earn points", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_events system="system_name" shared="true" force="true" header="your header" text="your custom text"]',
								'description' =>  __("This shortcode shows to customers the actions to perform in order to earn points.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'system',
										'desc' => __("The points and rewards system you want to display. You can find this value in <strong>WooRewards → Points and Rewards</strong>, in the <b>Shortcode Attribute</b> column. If you don’t set this value, nothing will be displayed.", 'woorewards-pro'),
									),
									array(
										'option' => 'shared',
										'desc' => __("If systems share the same points pool, you can show the rewards of all shared systems together. <strong>Warning</strong> : This only works if systems use the same type (Standard or Leveling).", 'woorewards-pro'),
									),
									array(
										'option' => 'force',
										'desc' => __("If set, the points will be shown even if the user currently doesn’t have access to the points and rewards system.", 'woorewards-pro'),
									),
									array(
										'option' => 'header',
										'desc' => __(" The text displayed over the earning methods list.", 'woorewards-pro'),
									),
									array(
										'option' => 'text',
										'desc' => __("This text is set to explain to customers what the information displayed is about.", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_events',
							)
						),
						'badges'    => array(
							'id' => 'lws_woorewards_sc_badges',
							'title' => __("Badges", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[lws_badges header=”<my Own Title>” display=”all|owned”]',
								'description' =>  __("This shortcode shows badges to customers.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the badges.", 'woorewards-pro'),
									),
									array(
										'option' => 'display',
										'desc' => __("Select if you want to show all existing badges (all) or only the ones owned by the customer (owned)", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_badges',
							)
						),
						'achievements'    => array(
							'id' => 'lws_woorewards_sc_achievements',
							'title' => __("Achievements", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[lws_achievements header=”<my Own Title>” display=”all|owned”]',
								'description' =>  __("This shortcode shows existing achievements.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the achievements.", 'woorewards-pro'),
									),
									array(
										'option' => 'display',
										'desc' => __("Select if you want to show all existing achievements (all) or only the ones unlocked by the customer (owned)", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_achievements',
							)
						),
						'social'    => array(
							'id' => 'lws_woorewards_sc_social',
							'title' => __("Social Share", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[lws_social_share header=“your own header” text=”your text” url=“specific url”]',
								'description' =>  __("This shortcode shows to customers the woocommerce coupons they currently have.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the coupons.", 'woorewards-pro'),
									),
									array(
										'option' => 'text',
										'desc' => __("The text displayed before the share buttons.", 'woorewards-pro'),
									),
									array(
										'option' => 'url',
										'desc' => __("By default, the widget shares the url of the page it’s displayed on. You can override that setting by setting an url in this option.", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'sty_widgets'), \admin_url('admin.php'))) . '#lws_group_targetable_social_share',
							)
						),
					)
				),
				'woocommerce' => array(
					'id'	=> 'woocommerce',
					'title'	=> __('WooCommerce Shortcodes', 'woorewards-pro'),
					'icon'	=> 'lws-icon-cart-2',
					'text'	=> __("In this section, you will find shortcodes specific to WooCommerce.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/shortcodes/woocommerce-shortcodes/'),
					'fields' => array(
						'user_loyalties'    => array(
							'id' => 'lws_woorewards_sc_user_loyalties',
							'title' => __("Loyalty and Rewards", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_user_loyalties]',
								'description' =>  __("This shortcode shows all loyalty and rewards information visible in WooCommerce's 'My Account' page.", 'woorewards-pro'),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'wc_settings'), \admin_url('admin.php'))) . '#lws_group_targetable_myaccountlarview',
							)
						),
						'cart_coupons'    => array(
							'id' => 'lws_woorewards_sc_cart_coupons',
							'title' => __("Cart Coupons", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_cart_coupons_view]',
								'description' =>  __("This shortcode shows coupons owned by the user and proposes a button to apply them on the cart.", 'woorewards-pro'),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'wc_settings'), \admin_url('admin.php'))) . '#lws_group_targetable_cartcouponsview',
							)
						),
						'cart_preview'    => array(
							'id' => 'lws_woorewards_sc_cart_preview',
							'title' => __("Cart Points Preview", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_cart_points_preview]',
								'description' =>  __("This shortcode shows the points the customers can get if he validates his actual cart.", 'woorewards-pro'),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'wc_settings'), \admin_url('admin.php'))) . '#lws_group_targetable_cartpointspreview',
							)
						),
						'product_preview'    => array(
							'id' => 'lws_woorewards_sc_product_previews',
							'title' => __("Product Points Preview", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[wr_product_points_preview id="product_id"]',
								'description' =>  __("This shortcode shows the points the customers can get if he purchases the actual product.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'id',
										'desc' => __("The product id. If not specified, the shortcode will try to find the actual product from the page.", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.appearance', 'tab' => 'wc_settings'), \admin_url('admin.php'))) . '#lws_group_targetable_productpointspreview',
							)
						),
					)
				),
				'sponsorship' => array(
					'id'	=> 'woocommerce',
					'title'	=> __('Sponsorship Shortcodes', 'woorewards-pro'),
					'icon'	=> 'lws-icon-handshake',
					'text'	=> __("In this section, you will find shortcodes relative to sponsorship.", 'woorewards-pro'),
					'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/shortcodes/sponsorship-shortcodes/'),
					'fields' => array(
						'email_sponsorship'    => array(
							'id' => 'lws_woorewards_sc_email_sponsorship',
							'title' => __("Email Sponsorthip", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[lws_sponsorship header="your header" button="Send" unlogged="true"]',
								'description' =>  __("This shortcode shows to customers a sponsorship email form.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the email sponsorship.", 'woorewards-pro'),
									),
									array(
										'option' => 'button',
										'desc' => __("The text displayed in the Submit button", 'woorewards-pro'),
									),
									array(
										'option' => 'unlogged',
										'desc' => __('(Optional) If set, unlogged users will be able to use the email sponsorship.', 'woorewards-pro'),
									)
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.settings', 'tab' => 'sp_settings'), \admin_url('admin.php'))) . '#lws_group_targetable_sponsor_widget_style',
							)
						),
						'link_sponsorship'    => array(
							'id' => 'lws_woorewards_sc_link_sponsorship',
							'title' => __("Sponsorthip Link", 'woorewards-pro'),
							'type' => 'shortcode',
							'extra' => array(
								'shortcode' => '[lws_sponsorship_link header="your header" display="both|qrcode|link" url="your_url"]',
								'description' =>  __("This shortcode shows to customers a sponsorship/referral link or QR Code.", 'woorewards-pro'),
								'options'   => array(
									array(
										'option' => 'header',
										'desc' => __("The text displayed before the sponsorship link or QR Code.", 'woorewards-pro'),
									),
									array(
										'option' => 'display',
										'desc' => __("Select the display type you want to show to customers : ", 'woorewards-pro')
											. "<ul><li><strong>both</strong> : " . __("QR Code and Link will be displayed", 'woorewards-pro') . "</li>"
											. "<li><strong>qrcode</strong> : " . __("Only the QR Code will be displayed", 'woorewards-pro') . "</li>"
											. "<li><strong>link</strong> : " . __("Only the Link will be displayed", 'woorewards-pro') . "</li></ul>",
									),
									array(
										'option' => 'url',
										'desc' => __("By default, the shortcode shares the url of the page it’s displayed on. You can override that setting by setting an url in this option.", 'woorewards-pro'),
									),
								),
								'style_url' => \esc_attr(\add_query_arg(array('page' => LWS_WOOREWARDS_PAGE . '.settings', 'tab' => 'sp_settings'), \admin_url('admin.php'))) . '#lws_group_targetable_referral',
							)
						),
					)
				),
			)
		);
		if (\get_option('lws_woorewards_enable_leaderboard'))
		{
			$shortcodesTab['groups']['shortcodes']['fields']['leaderboard'] = array(
				'id' => 'lws_woorewards_sc_leaderboard',
				'title' => __("Leaderboard", 'woorewards-pro'),
				'type' => 'shortcode',
				'extra' => array(
					'shortcode' => "[wr_leaderboard system='system_name' count='15']",
					'description' =>  __("This shortcode displays a leaderboard of your customers for a specific Points and Rewards System", 'woorewards-pro'),
					'options'   => array(
						array(
							'option' => 'system',
							'desc' => __("The points and rewards system for which you want to display the leaderboard. You can find this value in <strong>WooRewards → points and rewards systems</strong>, in the <b>Shortcode Attribute</b> column.", 'woorewards-pro'),
						),
						array(
							'option' => 'count',
							'desc' => __("(Optional) The number of rows displayed. Default is 15.", 'woorewards-pro'),
						),
						array(
							'option' => 'columns',
							'desc' => array(
								array(
									'tag' => 'p', 'join' => '<br/>',
									__("(Optional) The Columns to display (comma separated). <b>The order in which you specify the columns will be the grid columns order</b>.", 'woorewards-pro'),
									__(" If not specified, the leaderboard will display the rank, user nickname and points total of the users.", 'woorewards-pro'),
									__(" Here are the different options available :", 'woorewards-pro'),
								),
								array(
									'tag' => 'ul',
									array(
										"rank :",
										__("The user's rank in the leadeboard.", 'woorewards-pro'),
									), array(
										"user_nickname :",
										__("The user's display name.", 'woorewards-pro'),
									), array(
										"points :",
										__("The user's points in the points and rewards system.", 'woorewards-pro'),
									), array(
										"badges :",
										__("The badges owned by the customer. This will display the badges images and their titles on mouseover.", 'woorewards-pro'),
									), array(
										"last_badge :",
										__("The last badge earned by the customer. This will display the badge image and its title on mouseover.", 'woorewards-pro'),
									), array(
										"achievements :",
										__("The achievements unlocked by the customer. This will display the unlocked badges images and their titles on mouseover.", 'woorewards-pro'),
									), array(
										"user_title :",
										__("Displays the user title if he earned one.", 'woorewards-pro'),
									), array(
										"title_date :",
										__("Displays when the user earned his/her title.", 'woorewards-pro'),
									)
								)
							),
						),
						array(
							'option' => 'columns_headers',
							'desc' => __("(Optional) The column headers (comma separated). <b>Must be specified if you specified the columns option</b>. The headers must respect the same order than the ones of the previous option.", 'woorewards-pro'),
						),
						array(
							'option' => 'badges_ids',
							'desc' => __("(Optional) Restriction to specific achievements (comma separated). By default, all achievements can be displayed in the relevant columns. You can restrict that to a specific list of achievements.", 'woorewards-pro'),
						),
						array(
							'option' => 'achievement_ids',
							'desc' => __("(Optional) The number of rows displayed. Default is 15.", 'woorewards-pro'),
						),
					),
				)
			);
		};
		return $shortcodesTab;
	}

	function getEmailsTab()
	{
		$emailsTab = $this->standardPages['wr_appearance']['tabs']['sty_mails'];
		$emailsTab['vertnav'] = true;
		$emailsTab['groups']['wr_achieved'] = array_merge($emailsTab['groups']['wr_achieved'], array('icon' => 'lws-icon-trophy', 'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/emails/achievement-email/')));
		$emailsTab['groups']['wr_available_unlockables'] = array_merge($emailsTab['groups']['wr_available_unlockables'], array('icon' => 'lws-icon-questionnaire', 'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/emails/reward-choice/')));
		$emailsTab['groups']['couponreminder'] = array_merge($emailsTab['groups']['couponreminder'], array('icon' => 'lws-icon-coupon', 'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/emails/reward-expiration/')));
		$emailsTab['groups']['pointsreminder'] = array_merge($emailsTab['groups']['pointsreminder'], array('icon' => 'lws-icon-calendar', 'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/emails/points-expiration/')));
		$emailsTab['groups']['wr_sponsored'] = array_merge($emailsTab['groups']['wr_sponsored'], array('icon' => 'lws-icon-users-mm', 'extra' => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/emails/sponsorship-email/')));
		return $emailsTab;
	}

	function getPopupTab()
	{
		$popupTab = array(
			'id'	=> 'reward_popup',
			'title'	=>  __("Popups", 'woorewards-pro'),
			'icon'	=> 'lws-icon-window-add',
			'groups' => array(
				array(
					'id'     => 'claim',
					'icon'	=> 'lws-icon-window-add',
					'title'  => __("Reward Popup", 'woorewards-pro'),
					'text'   => __("Defines the popup options when a user unlocks a reward.", 'woorewards-pro'),
					'fields' => array(
						array(
							'id'    => 'lws_wr_rewardclaim_popup_disable',
							'title' => __("Disable the reward popup", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => '',
								'class' => 'lws_checkbox',
							)
						),
						'claim' => array(
							'id'    => 'lws_woorewards_reward_claim_page',
							'title' => __("Redirection page after a reward is unlocked", 'woorewards-pro'),
							'type'  => 'lacselect',
							'extra' => array(
								'predefined' => 'page',
								'tooltips' => __("When a customer clicks a reward redeem button, he will be redirected to that page.", 'woorewards-pro')
									. '<br/>' . __("If WooCommerce is activated, the default is the <b>Loyalty and Rewards</b> tab in the customer my-account frontend page. Otherwise, it is your home page", 'woorewards-pro')
							)
						),
						array(
							'id'    => 'lws_wr_rewardclaim_notice_with_rest',
							'title' => __("Show remaining available rewards after a reward is unlocked", 'woorewards-pro'),
							'type'  => 'box',
							'extra' => array(
								'default' => 'on',
								'class' => 'lws_checkbox',
								'tooltips' => __("When a customer clicks a reward redeem button, he will be redirected to a page with an unlock feedback.", 'woorewards-pro')
									. '<br/>' . __("That popup includes the rest of available rewards.", 'woorewards-pro')
							)
						),
						'popup' => array(
							'id' => 'lws_woorewards_lws_reward_claim',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'lws_reward_claim',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/rewardclaim.css',
								'subids' => array(
									'lws_woorewards_wc_reward_claim_title' => "WooRewards - Reward Claim Popup - Title",
									'lws_woorewards_wc_reward_claim_header' => "WooRewards - Reward Claim Popup - Header",
									'lws_woorewards_wc_reward_claim_stitle' => "WooRewards - Reward Claim Popup - Subtitle",
								),
								'help' =>  __("This popup will show when customers unlock a new reward.", 'woorewards-pro') . "<br/>"
									. __("It can show only the reward unlocked or also the rewards that can still be unlocked .", 'woorewards-pro')
							)
						),
					)
				),
				array(
					'id'     => 'freeproduct',
					'icon'	=> 'lws-icon-window-add',
					'title'  => __("Free Product Popup", 'woorewards-pro'),
					'text'   => __("Defines the popup options when a customer uses a free product coupon with multiple choices.", 'woorewards-pro'),
					'fields' => array(
						'popup' => array(
							'id' => 'lws_woorewards_free_product_template',
							'type' => 'stygen',
							'extra' => array(
								'purpose' => 'filter',
								'template' => 'free_product_template',
								'html' => false,
								'css' => LWS_WOOREWARDS_PRO_CSS . '/templates/freeproduct.css',
								'subids' => array(
									'lws_free_product_popup_title' => "WooRewards Free Product - title",
									'lws_free_product_popup_cancel' => "WooRewards Free Product - cancel button",
									'lws_free_product_popup_validate' => "WooRewards Free Product - validate button",
								),
								'help' =>  __("This popup will show when customers use a free product coupon.", 'woorewards-pro')
							)
						),
					)
				)
			)
		);
		return $popupTab;
	}

	/** @return pool name or false. */
	protected function guessCurrentPool($tabId)
	{
		$ref = false;
		$tab = isset($_REQUEST['tab']) ? trim($_REQUEST['tab']) : '';
		$tabPrefix = $tabId . '.wr_upool_';

		if (strpos($tab, $tabPrefix) === 0)
		{
			$ref = substr($tab, strlen($tabPrefix));
		}
		else if (defined('DOING_AJAX') && DOING_AJAX && isset($_REQUEST['action']) && $_REQUEST['action'] == 'lws_adminpanel_editlist')
		{
			$editlist = isset($_REQUEST['id']) ? trim($_REQUEST['id']) : '';
			foreach (array('UnlockableList-', 'EventList-') as $prefix)
			{
				if (0 === strpos($editlist, $prefix))
				{
					$ref = intval(substr($editlist, strlen($prefix)));
					break;
				}
			}
		}

		$guess = false;
		if ($ref)
		{
			if ($id = max(0, intval($ref)))
				$guess = \LWS\WOOREWARDS\Collections\Pools::instanciate()->load(array('p' => $id, 'deep' => true))->last();

			if (!$guess)
				$guess = \LWS\WOOREWARDS\Collections\Pools::instanciate()->load(array('name' => $ref, 'deep' => true))->last();
		}
		return $guess;
	}

	protected function getLoyaltyGroups($pool)
	{
		if (empty($pool))
		{
			return array('error' => array(
				'title' => __("Loading failure", 'woorewards-pro'),
				'text'  => __("Seems the points and rewards system does not exists. Try re-activate this plugin. If that problem persists, contact your administrator.", 'woorewards-pro')
			));
		}

		$settingsText = array(
			'cast' => 'p',
			array(
				__("General Settings are used to start or stop your points and rewards system, rename it or change some other basic options.", 'woorewards-pro'),
				__("If you're not sure how to use them, please refer to the dedicated documentation by clicking the book icon on the top right.", 'woorewards-pro'),
			),
			array('tag' => 'strong', __("Don't forget to start your points and rewards system when you've finished your settings.", 'woorewards-pro'))
		);

		$earningText = array(
			'cast' => 'p',
			__("Use this section to define what actions users or customers have to perform in order to earn points in this points and rewards system.", 'woorewards-pro'),
			array('tag' => 'strong', __("You can define as many actions as you want by clicking the 'Add' button.", 'woorewards-pro')),
		);

		$group = array(
			'earning'    => array(
				'id'      => 'wr_loyalty_earning',
				'class'   => 'half',
				'title'   => __("Points", 'woorewards-pro'),
				'image'   => LWS_WOOREWARDS_IMG . '/ls-earning.png',
				'color'   => '#38bebe',
				'text'    => $earningText,
				'extra'   => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/points/'),
				'editlist' => \lws_editlist(
					'EventList-' . $pool->getId(),
					\LWS\WOOREWARDS\Ui\Editlists\EventList::ROW_ID,
					new \LWS\WOOREWARDS\Ui\Editlists\EventList($pool),
					\LWS\Adminpanel\EditList::MDA
				)->setPageDisplay(false)->setCssClass('eventlist')->setRepeatHead(false),
			),
			'spending'   => $this->getPoolRewardsGroup($pool),
			'general'    => array(
				'id'     => 'wr_loyalty_general',
				'image'  => LWS_WOOREWARDS_IMG . '/ls-settings.png',
				'color'  => '#7958a5',
				'class'  => 'half',
				'title'  => __("General Settings", 'woorewards-pro'),
				'text'   => $settingsText,
				'extra'  => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/tutorials/points-and-rewards/'),
				'fields' => \apply_filters('lws_woorewards_admin_pool_general_settings', array(), $pool)
			),
			'expiration' => $this->getPoolPointExpirationGroup($pool),
			'pts_disp'   => $this->getPoolPointDisplayGroup($pool),
			'advanced'	 => $this->getPoolAdvancedGroup($pool),
		);

		return $group;
	}

	protected function getPoolAdvancedGroup($pool)
	{
		$asettingsText = array(
			'join' => '<br/><br/>',
			__("Advanced Settings are used to set special options to change the behavior of the points and rewards system. These options can have a serious impact on the points and rewards system.", 'woorewards-pro'),
			array(
				array('tag' => 'strong', __("Warning :", 'woorewards-pro')),
				__("Be sure of what you're doing before making modifications in this section.", 'woorewards-pro'),
			)
		);

		$group = array(
			'id'		=> 'wr_loyalty_asettings',
			'image'		=> LWS_WOOREWARDS_IMG . '/ls-asettings.png',
			'color'		=> '#6e9684',
			'class'		=> 'half',
			'title'		=> __("Advanced Settings", 'woorewards-pro'),
			'text'		=> $asettingsText,
			'extra'		=> array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/advanced-mechanisms/advanced-settings/'),
			'fields'	=> array(
				'stackid' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'stack',
					'title' => __("Point Reserve", 'woorewards-pro'),
					'type'  => 'lacselect',
					'extra' => array(
						'id'       => 'lws_wr_pool_pointstack',
						'class'    => 'lws-wr-pool-pointstack',
						'value'    => $pool->getRawStackId(),
						'ajax'     => 'lws_woorewards_pointstack_list',
						'source'   => array(
							array('value' => '', 'label' => sprintf('<i>%s</i>', __("&lt;Create a new reserve&gt;", 'woorewards-pro'))),
						),
						'mode' => 'select',
						'tooltips' => sprintf(
							'<span class="pointstack_help single">%s</span><span class="pointstack_help shared hidden"><span class="text">%s</span> <span class="list">&nbsp;</span></span>',
							__("This System uses its own points reserve.", 'woorewards-pro'),
							__("This System shares its Points between :", 'woorewards-pro')
						)
					)
				),
				'restricted' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'roles',
					'title' => __("Allowed Roles", 'woorewards-pro'),
					'type'  => 'lacchecklist',
					'extra' => array(
						'value'    => $pool->getOption('roles'),
						'ajax'     => 'lws_adminpanel_get_roles',
						'tooltips' => __("If set, only users with at least one of the selected roles can enjoy that points and rewards system. By default, a loyalty system is available for everybody.", 'woorewards-pro')
					)
				),
				'denied' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'denied_roles',
					'title' => __("Denied Roles", 'woorewards-pro'),
					'type'  => 'lacchecklist',
					'extra' => array(
						'value'    => $pool->getOption('denied_roles'),
						'ajax'     => 'lws_adminpanel_get_roles',
						'tooltips' => __("If set, users with at least one of the selected roles won't have access to the Points and Rewards System.", 'woorewards-pro')
					)
				),
			)
		);

		if ($pool->getOption('type') == \LWS\WOOREWARDS\Core\Pool::T_LEVELLING)
		{
			$bestLabel = __("Unlock Best Level <b>Only</b>", 'woorewards-pro');
			$bestOptions = array(
				'off'  => array('value' => 'off', 'label' => _x("Off", "best_unlock settings", 'woorewards-pro')),
				'on'   => array('value' => 'on', 'label' => _x("On", "best_unlock settings", 'woorewards-pro')),
			);

			$group['fields']['confiscation'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'confiscation',
				'title' => __("Lose rewards with points expiration", 'woorewards-pro'),
				'type'  => 'box',
				'extra' => array(
					'checked' => $pool->getOption('confiscation'),
					'class' => 'lws_checkbox',
					'tooltips' => __("After a points loss due to points expiration, the customer will have to earn all the rewards again. Ignored if points expiration isn’t set.", 'woorewards-pro')
				)
			);

			$group['fields']['clamp_level'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'clamp_level',
				'title' => __("One level at a time", 'woorewards-pro'),
				'type'  => 'box',
				'extra' => array(
					'checked' => $pool->getOption('clamp_level'),
					'default' => false,
					'class' => 'lws_checkbox',
					'tooltips' => __("If checked, customers can't earn more points than the points needed to reach the next level in one time.", 'woorewards-pro')
				)
			);

			$group['fields']['best_unlock'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'best_unlock',
				'title' => $bestLabel,
				'type'  => 'lacselect',
				'extra' => array(
					'value'    => $pool->getOption('best_unlock'),
					'source'   => $bestOptions,
					'id'       => 'reward_best_unlock',
					'mode'     => 'select',
					'default'  => 'off',
					'tooltips' => \lws_array_to_html(array(
						'tag' => 'ul',
						array(
							__("Off :", 'woorewards-pro'),
							__("All levels will be unlocked, even if the user earns enough points in one time to unlock several levels.", 'woorewards-pro'),
						),
						array(
							__("On :", 'woorewards-pro'),
							__("Only the best level will be unlocked. If a user earns enough points to unlock several levels, only the highest will be unlocked", 'woorewards-pro'),
						),
					)),
				)
			);
		}



		return $group;
	}

	protected function getPoolPointExpirationGroup($pool)
	{
		$expirationText = array(
			'join' => '<br/><br/>',
			__("You have access to different points expiration methods. To get more information on those methods, don't hesitate to take a look at the dedicated documentation. Simply click the book Icon on the top right.", 'woorewards-pro'),
			array(
				array('tag' => 'strong', __("Warning :", 'woorewards-pro')),
				__("You should only use one expiration method in your points and rewards system", 'woorewards-pro'),
			)
		);

		return array(
			'id'		=> 'wr_loyalty_expiration',
			'image'		=> LWS_WOOREWARDS_IMG . '/ls-calendar.png',
			'color'		=> '#a4489a',
			'class'		=> 'half',
			'title'		=> __("Points Expiration", 'woorewards-pro'),
			'text'		=> $expirationText,
			'extra'		=> array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/advanced-mechanisms/points-expiration/'),
			'fields'	=> array(
				'lifetime' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'point_timeout',
					'title' => __("Points expiration for inactivity", 'woorewards-pro'),
					'type'  => 'duration',
					'extra' => array(
						'value' => $pool->getOption('point_timeout')->toString(),
						'help' => __("Defines if customers lose their points after an inactivity period", 'woorewards-pro')
					)
				),
				'transactional_expiry' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'transactional_expiry',
					'title' => __("Transactional Points expiration", 'woorewards-pro'),
					'type'  => 'woorewards_periodic_trigger',
					'extra' => array(
						'value' => $pool->getOption('transactional_expiry'),
						'help' => sprintf(__("Defines if customers lose unused points periodically. Please read the %s for settings explanation.", 'woorewards-pro'), "<a href='https://plugins.longwatchstudio.com/docs/woorewards-4/points-and-rewards-systems/' target='_blank'>" . __("documentation", 'woorewards-pro') . "</a>"),
					)
				)
			)
		);
	}

	protected function getPoolPointDisplayGroup($pool)
	{
		$currencyText = array(
			'join' => '<br/><br/>',
			__("You can change how points are displayed to customers. You can either set a text or an image.", 'woorewards-pro'),
			array(
				array('tag' => 'strong', __("Warning :", 'woorewards-pro')),
				__("If you use multiple languages, labels won't be translated with po/mo files. However, it's possible with WPML.", 'woorewards-pro'),
			),
		);

		return array(
			'id'       => 'wr_pts_disp',
			'image'		=> LWS_WOOREWARDS_IMG . '/ls-currency.png',
			'color'		=> '#a67c52',
			'class'		=> 'half',
			'title'    => __("Points Currency", 'woorewards-pro'),
			'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/11-advanced-mechanisms/points-currency/'),
			'text' 	   => $currencyText,
			'fields'   => array(
				'point_name' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'point_name_singular',
					'title' => __("Point display name", 'woorewards-pro'),
					'type'  => 'text',
					'extra' => array(
						'value' => $pool->getOption('point_name_singular'),
						'placeholder' => __("Point", 'woorewards-pro'), //\LWS_WooRewards::getPointSymbol(1),
						'tooltips' => __("Point unit shown to the user.", 'woorewards-pro'),
					)
				),
				'points_name' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'point_name_plural',
					'title' => __("Points display name (plural)", 'woorewards-pro'),
					'type'  => 'text',
					'extra' => array(
						'value' => $pool->getOption('point_name_plural'),
						'placeholder' => __("Points", 'woorewards-pro'), //\LWS_WooRewards::getPointSymbol(2),
						'tooltips' => __("(Optional) The singular form is used if plural is not set.", 'woorewards-pro'),
					)
				),
				'point_sym' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'symbol',
					'title' => __("Point symbol", 'woorewards-pro'),
					'type'  => 'media',
					'extra' => array(
						'value' => $pool->getOption('symbol'),
						'tooltips' => __("If you set an image, it will replace the above labels.", 'woorewards-pro'),
					)
				),
				'point_format' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'point_format',
					'title' => __("Point name position", 'woorewards-pro'),
					'type'  => 'lacselect',
					'extra' => array(
						'value' => $pool->getOption('point_format'),
						'mode' => 'select',
						'source' => array(
							array('value' => '%1$s %2$s', 'label' => _x("Right", 'Point name position', 'woorewards-pro')),
							array('value' => '%2$s %1$s', 'label' => _x("Left", 'Point name position', 'woorewards-pro')),
						),
					)
				),
				'thousand_sep' => array(
					'id'    => self::POOL_OPTION_PREFIX . 'thousand_sep',
					'title' => __("Thousand Separator", 'woorewards-pro'),
					'type'  => 'text',
					'extra' => array(
						'value' => $pool->getOption('thousand_sep'),
						'tooltips' => __("(Optional) The thousand separator when displaying big numbers.", 'woorewards-pro'),
					)
				),
			),
		);
	}

	protected function getPoolRewardsGroup($pool)
	{
		$group = array(
			'id'     => 'lws_wr_spending_system',
			'class'  => 'half',
			'image'  => LWS_WOOREWARDS_IMG . '/ls-gift.png',
			'color'  => '#526981',
			'title'  => '',
			'text'   => '',
			'extra'  => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards-4/rewards/'),
			'fields' => array(),
		);

		$rewards = array(
			'id'    => 'rewards',
			'type'  => 'editlist',
			'extra' => array(
				'editlist' => \lws_editlist(
					'UnlockableList-' . $pool->getId(),
					\LWS\WOOREWARDS\Ui\Editlists\UnlockableList::ROW_ID,
					new \LWS\WOOREWARDS\Ui\Editlists\UnlockableList($pool),
					\LWS\Adminpanel\EditList::MDA
				)->setPageDisplay(false)->setGroupBy($this->getGroupByLevelSettings($pool))->setCssClass('unlockablelist')->setRepeatHead(false),
			),
		);

		if (\LWS\WOOREWARDS\Core\Pool::T_LEVELLING == $pool->getOption('type'))
		{
			$group['title'] = __('Levels and Rewards', 'woorewards-pro');
			$group['text'] = array(
				'join' => '<br/>',
				array(
					__("In a leveling system, you must ", 'woorewards-pro'),
					array('tag' => 'strong', __("create levels first", 'woorewards-pro')),
				),
				array(
					__("After creating a level, you can add one or more rewards to the level.", 'woorewards-pro'),
					__("If you're not sure how to set up levels and rewards, please refer to the dedicated documentation by clicking the book icon on the top right.", 'woorewards-pro'),
				),
				array(
					array('tag' => 'strong', __("You can define as many levels and rewards as you want", 'woorewards-pro')),
				),
			);

			$group['fields']['rewards'] = $rewards;
		}
		else // T_STANDARD
		{
			$group['title'] = __('Rewards', 'woorewards-pro');
			$group['text'] = array(
				__("In a standard system, you have to choose how customers will spend their points.", 'woorewards-pro'),
				array('tag' => 'strong', __("They can either spend them directly on the cart to get an immediate discount or you can setup various rewards they buy with their points.", 'woorewards-pro')),
				__("If you're not sure how to set up rewards, please refer to the dedicated documentation by clicking the book icon on the top right.", 'woorewards-pro'),
			);

			$group['fields']['mode'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'direct_reward_mode',
				'type'  => 'box',
				'title' => __("Rewards Type", 'woorewards-pro'),
				'extra' => array(
					'id'      => 'direct_reward_mode',
					'class'   => 'lws_switch',
					'value'   => $pool->getOption('direct_reward_mode'),
					'data'    => array(
						'left' => __("Rewards", 'woorewards-pro'),
						'right' => __("Points on Cart", 'woorewards-pro'),
						'colorleft' => '#425981',
						'colorright' => '#5279b1',
					),
				)
			);

			$group['fields']['rate'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'direct_reward_point_rate',
				'type'  => 'text',
				'title' => sprintf(__("Point Value in %s", 'woorewards-pro'), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?'),
				'extra' => array(
					'value'   => $pool->getOption('direct_reward_point_rate'),
					'help' => __("Each point spent on the cart will decrease the order total of that value", 'woorewards-pro')
				),
				'require' => array('selector' => '#direct_reward_mode', 'value' => 'on'),
			);

			$group['fields']['max_percent'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'direct_reward_max_percent_of_cart',
				'title' => __("Max percentage of cart", 'woorewards-pro'),
				'type'  => 'text',
				'extra' => array(
					'value'       => $pool->getOption('direct_reward_max_percent_of_cart'),
					'pattern'     => '\d*(.|,)\d*',
					'placeholder' => '%',
					'help'        => __("The maximum amount a customer can spend in a single cart will be limited to the percentage of the payable total. Leave blank for no limit.", 'woorewards-pro'),
				),
				'require' => array('selector' => '#direct_reward_mode', 'value' => 'on'),
			);

			$group['fields']['min_grandtotal'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'direct_reward_total_floor',
				'title' => __("Lower Cart Limit", 'woorewards-pro'),
				'type'  => 'text',
				'extra' => array(
					'value'       => $pool->getOption('direct_reward_total_floor'),
					'pattern'     => '\d*(.|,)\d*',
					'placeholder' => \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '',
					'help'        => __("If set, customers can't use their points to discount the cart below that limit. Leave empty for no limit.", 'woorewards-pro'),
				),
				'require' => array('selector' => '#direct_reward_mode', 'value' => 'on'),
			);

			$group['fields']['min_subtotal'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'direct_reward_min_subtotal',
				'title' => __("Minimum Cart Amount", 'woorewards-pro'),
				'type'  => 'text',
				'extra' => array(
					'value'       => $pool->getOption('direct_reward_min_subtotal'),
					'pattern'     => '\d*(.|,)\d*',
					'placeholder' => \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '',
					'help'        => __("Set a minimum cart amount under which customers can't use their points on the cart. Once the cart total is above that value, customers will be able to use their points. Leave empty for no minimum.", 'woorewards-pro'),
				),
				'require' => array('selector' => '#direct_reward_mode', 'value' => 'on'),
			);

			if ($pool->getOption('type') != \LWS\WOOREWARDS\Core\Pool::T_LEVELLING)
			{
				$bestLabel = __("Automatic Rewards Redeem", 'woorewards-pro');

				$bestOptions = array(
					'off'  => array('value' => 'off', 'label' => _x("Off", "best_unlock settings", 'woorewards-pro')),
					'on'   => array('value' => 'on', 'label' => _x("Unlock best reward only", "best_unlock settings", 'woorewards-pro')),
					'loop' => array('value' => 'and_loop', 'label' => _x("Unlock best reward first", "best_unlock settings", 'woorewards-pro')),
					'raz'  => array('value' => 'use_all_points', 'label' => _x("Unlock best reward and reset points", "best_unlock settings", 'woorewards-pro')),
				);
				$group['fields']['best_unlock'] = array(
					'id'    => self::POOL_OPTION_PREFIX . 'best_unlock',
					'title' => $bestLabel,
					'type'  => 'lacselect',
					'extra' => array(
						'value'    => $pool->getOption('best_unlock'),
						'source'   => $bestOptions,
						'id'       => 'reward_best_unlock',
						'mode'     => 'select',
						'default'  => 'off',
						'tooltips' => \lws_array_to_html(array(
							'tag' => 'ul',
							array(
								__("Off :", 'woorewards-pro'),
								__("The customer has to manually redeem the rewards. A mail is sent each time he earns enough points for at least one of them.", 'woorewards-pro'),
							),
							array(
								__("Best only :", 'woorewards-pro'),
								__("Only the most expensive reward or the highest level the user can afford will be unlocked.", 'woorewards-pro'),
							),
							array(
								__("Best first :", 'woorewards-pro'),
								__("Rewards are unlocked as long as customer can afford it, starting by the most expensive reward (The same reward can be unlocked several time).", 'woorewards-pro'),
							),
							array(
								__("Best and Reset :", 'woorewards-pro'),
								__("Same as 'Best only' but consume all customer's points.", 'woorewards-pro'),
							),
						)),
					),
					'require' => array('selector' => '#direct_reward_mode', 'value' => ''),
				);

			}

			$rewards['require'] = array('selector' => '#direct_reward_mode', 'value' => '');
			$group['fields']['rewards'] = $rewards;
		}

		return $group;
	}

	protected function getGroupByLevelSettings($pool)
	{
		$groupBy = array(
			'key'       => 'cost',
			'activated' => ($pool->getOption('type') == \LWS\WOOREWARDS\Core\Pool::T_LEVELLING),
			'add'       => __("Add level", 'woorewards-pro'),
		);
		$labels = array(
			'group_value' => __("Untitled", 'woorewards-pro'),
			'group_title' => _x("Level Title", "Level Threshold Title edit", 'woorewards-pro'),
			'group_point' => _x("Points Threshold", "edit", 'woorewards-pro'),
			'form_title'  => \esc_attr(__("Title is required.", 'woorewards-pro')),
			'form_point'  => \esc_attr(__("Cost must be a number greater than zero.", 'woorewards-pro')),
			'title_title' => _x("Level Title", "Level Threshold Title edit", 'woorewards-pro'),
			'title_point' => _x("Points Threshold", "edit", 'woorewards-pro'),
		);
		$groupBy['head'] = <<<EOT
<div class='lws-wr-levelling-node-head'>
	<div class='lws-wr-levelling-node-item grouped_title'>
		<div class='lws-wr-levelling-node-value'><span data-name='grouped_title'>{$labels['group_value']}</span></div>
		<div class='lws-wr-levelling-node-label'>{$labels['group_title']}</div>
	</div>
	<div class='lws-wr-levelling-node-item cost'>
		<div class='lws-wr-levelling-node-value'><span data-name='cost'>1</span></div>
		<div class='lws-wr-levelling-node-label'>{$labels['group_point']}</div>
	</div>
</div>
EOT;
		$groupBy['form'] = <<<EOT
<div class='lws-wr-levelling-node-form'>
	<div class='lws-wr-levelling-node-item grouped_title'>
		<div class='lws-wr-levelling-node-value'><input type='text' class='lws-input lws-wr-title-input' name='grouped_title' data-pattern='[^\\s]+' data-pattern-title='{$labels['form_title']}'/></div>
		<div class='lws-wr-levelling-node-label'>{$labels['title_title']}</div>
	</div>
	<div class='lws-wr-levelling-node-item cost'>
		<div class='lws-wr-levelling-node-value'><input name='cost' class='lws-input lws-wr-cost-input' type='text' data-pattern='^\\d*[1-9]\\d*$' data-pattern-title='{$labels['form_point']}'/></div>
		<div class='lws-wr-levelling-node-label'>{$labels['title_point']}</div>
	</div>
</div>
EOT;
		return $groupBy;
	}

	function poolGeneralSettings($fields, \LWS\WOOREWARDS\Core\Pool $pool)
	{
		if (empty($pool->getId()))
		{
			$fields['type'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'type',
				'type'  => 'select',
				'title' => __("Behavior", 'woorewards-pro'),
				'extra' => array(
					'id'      => 'lws-wr-pool-option-type',
					'value'   => $pool->getOption('type'),
					'notnull' => true,
					'options' => array(
						'standard'  => _x("Standard", "Pool Type/behavior", 'woorewards-pro'),
						'levelling' => _x("Levelling", "Pool Type/behavior", 'woorewards-pro')
					),
					'help' => '<ul><li>' . __("<i>Standard behavior</i>: customers can spend points to buy rewards", 'woorewards-pro')
						. '</li><li>' . __("<i>Levelling behavior</i>: rewards are automatically granted to customers since they have enough points (points are never spent).", 'woorewards-pro')
						. '</li></ul>'
				)
			);
		}
		$happening = $pool->getOption('happening') ? true : false;
		$fields['lifestyle'] = 	array(
			'id'    => self::POOL_OPTION_PREFIX . 'happening',
			'type'  => 'box',
			'title' => __("System Type", 'woorewards-pro'),
			'extra' => array(
				'id'      => 'lws_woorewards_system_type',
				'class'   => 'lws_switch',
				'checked' => $happening,
				'data'    => array(
					'left' => __("Permanent", 'woorewards-pro'),
					'right' => __("Event", 'woorewards-pro'),
					'colorleft' => '#7958a5',
					'colorright' => '#5279b1',
				),
			)
		);


		if ($pool->isDeletable())
		{
			$date = $pool->getOption('period_start');
			$fields['period_start'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'period_start',
				'type'  => 'input',
				'title' => __("Start Date", 'woorewards-pro'),
				'extra' => array(
					'id'      => 'lws-wr-pool-option-period-begin',
					'type'  => 'date',
					'value' => empty($date) ? '' : $date->format('Y-m-d'),
					'help'  => __("Before that date, the points and rewards system is disabled but customer can see it.", 'woorewards-pro')
				),
				'require' => array('selector' => '#lws_woorewards_system_type', 'value' => 'on'),
			);

			if ($pool->getOption('type') != \LWS\WOOREWARDS\Core\Pool::T_LEVELLING)
			{
				$date = $pool->getOption('period_mid');
				$fields['period_mid'] = array(
					'id'    => self::POOL_OPTION_PREFIX . 'period_mid',
					'type'  => 'input',
					'title' => __("Point earning end", 'woorewards-pro'),
					'extra' => array(
						'id'      => 'lws-wr-pool-option-period-mid',
						'type'  => 'date',
						'value' => empty($date) ? '' : $date->format('Y-m-d'),
						'help'  => __("After that date, customers can no longer earn points. But they still can spend them for rewards.", 'woorewards-pro')
					),
					'require' => array('selector' => '#lws_woorewards_system_type', 'value' => 'on'),
				);
			}

			$date = $pool->getOption('period_end');
			$fields['period_end'] = array(
				'id'    => self::POOL_OPTION_PREFIX . 'period_end',
				'type'  => 'input',
				'title' => __("End Date", 'woorewards-pro'),
				'extra' => array(
					'id'      => 'lws-wr-pool-option-period-end',
					'type'  => 'date',
					'value' => empty($date) ? '' : $date->format('Y-m-d'),
					'help'  => __("After that date, the points and rewards system will be disabled but customer can see it. Customers keep their remaining points but cannot use them anymore.", 'woorewards-pro')
				),
				'require' => array('selector' => '#lws_woorewards_system_type', 'value' => 'on'),
			);
		}

		return $fields;
	}

	function userspointsFilters($filters)
	{
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsrangefilter.php';
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsactivityfilter.php';

		$filters = array_merge(array(
			'range' => new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsRangeFilter('range'),
			'activity' => new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsActivityFilter('activity')
		), $filters);

		if (!empty(\get_option('lws_woorewards_manage_badge_enable', 'on')))
		{
			require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsbadgefilter.php';
			require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsbadgeassignbulkaction.php';
			require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsbadgeremovebulkaction.php';

			$filters = array_merge(array(
				'badge' => new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsBadgeFilter('badge'),
			), $filters);

			$filters['badge_add'] = new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsBadgeAssignBulkAction('badge_add');
			$filters['badge_rem'] = new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsBadgeRemoveBulkAction('badge_rem');
		}

		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsbulkaction.php';
		$filters['points_add'] = new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsBulkAction('points_add');

		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsunlockablesba.php';
		$filters['u_redeem'] = new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsUnlockablesBA('u_redeem');
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/ui/editlists/userspointsconfiscationba.php';
		$filters['u_revoke'] = new \LWS\WOOREWARDS\PRO\Ui\Editlists\UsersPointsConfiscationBA('u_revoke');

		return $filters;
	}

	function getSystemPage()
	{
		$system = $this->standardPages['wr_system'];
		$system['tabs']['data_management']['groups'] = array_merge($system['tabs']['data_management']['groups'], $this->getMigrationGroups());

		if (isset($system['tabs']['data_management']['groups']['delete']))
		{
			// move at last pos
			$delete = $system['tabs']['data_management']['groups']['delete'];
			unset($system['tabs']['data_management']['groups']['delete']);
			$system['tabs']['data_management']['groups']['delete'] = $delete;
		}
		return $system;
	}

	function getMigrationGroups()
	{
		require_once LWS_WOOREWARDS_PRO_INCLUDES . '/migration/exportmethods.php';
		$exports = new \LWS\WOOREWARDS\PRO\Migration\ExportMethods();

		$groups = array(
			'export_wr' => array(
				'id' => 'wr_export_points_wr',
				'icon'	 => 'lws-icon-migration',
				'class'	=> 'half',
				'title' => __("Export Points from WooRewards", 'woorewards-pro'),
				'text' => __("Select a points and rewards system to export the users points from that system.", 'woorewards-pro'),
				'fields' => array(
					'pool' => array(
						'id'    => 'woorewards-pro' . '_from_pool',
						'type'  => 'lacselect',
						'title' => __("points and rewards system", 'woorewards-pro'),
						'extra' => array(
							'class'    => 'lws-ignore-confirm',
							'maxwidth' => '400px',
							'gizmo'    => true,
							'ajax'     => 'lws_woorewards_pool_list',
						)
					),
					'export' => array(
						'id'    => 'export-wr',
						'type'  => 'button',
						'title' => __("Export", 'woorewards-pro'),
						'extra' => array(
							'link' => array('ajax' => 'woorewards-pro' . '-export-wr'),
						)
					),
				)
			),
			'export_other' => array(
				'id' 	=> 'wr_export_points_other',
				'icon'	=> 'lws-icon-migration',
				'class'	=> 'half',
				'title' => __("Export from other plugins", 'woorewards-pro'),
				'text'	=> sprintf(
					'%s<br/><strong>%s</strong>',
					__("If you're migrating from another loyalty plugin, you can export the users points from the other plugin and import them into WooRewards.", 'woorewards-pro'),
					__("The other plugin needs to be installed and activte for this procedure to work.", 'woorewards-pro')
				),
				'fields' => array(
					'meta' => array(
						'id'    => 'woorewards-pro' . '_from_meta',
						'type'  => 'lacselect',
						'title' => __("Loyalty Plugin or Meta Key", 'woorewards-pro'),
						'extra' => array(
							'class' => 'lws-ignore-confirm',
							'value' => '—',
							'maxwidth' => '400px',
							'allownew' => 'on',
							'source' => $exports->getMethods(),
							'gizmo' => true,
							'tooltips' => __("Do not change that value if you are not sure about what you are doing.", 'woorewards-pro'),
						)
					),
					'arg' => array(
						'id'    => 'woorewards-pro' . '_with_arg',
						'type'  => 'lacselect',
						'title' => __("Some Plugin need extra arguments", 'woorewards-pro'),
						'extra' => array(
							'class' => 'lws-ignore-confirm',
							'value' => '—',
							'allownew' => 'on',
							'maxwidth' => '400px',
							'source' => $exports->getArguments(),
							'gizmo' => true,
							'tooltips' => __("Main purpose is for plugins that support several point pools. If the plugin is not listed here, it does not need an extra argument.", 'woorewards-pro'),
						)
					),
					'export' => array(
						'id'    => 'export-points',
						'type'  => 'button',
						'title' => __("Export", 'woorewards-pro'),
						'extra' => array(
							'link' => array('ajax' => 'woorewards-pro' . '-export-points'),
						)
					),
				)
			),
			'import' => array(
				'id' => 'wr_import_points',
				'icon'	=> 'lws-icon-cloud-download-93',
				'title' => __("Import Points", 'woorewards-pro'),
				'text'  => implode('<br/>', array(
					__("Select the exported file, then click on «Import».", 'woorewards-pro'),
					__("The Import process does <b>not</b> generate any reward.", 'woorewards-pro'),
				)),
				'fields' => array(
					'round' => array(
						'id'    => 'woorewards-pro' . '_rounding',
						'type'  => 'lacselect',
						'title' => __("Round imported points", 'woorewards-pro'),
						'extra' => array(
							'default' => 'floor',
							'maxwidth' => '400px',
							'mode'	=> 'select',
							'tooltips' => __("WooRewards only support integer points", 'woorewards-pro'),
							'source' => array(
								array('value' => 'floor', 'label' => __("Round fractions down", 'woorewards-pro')),
								array('value' => 'ceil',  'label' => __("Round fractions up", 'woorewards-pro')),
								array('value' => 'half_up', 'label' => __("Round to nearest integer, half way round up", 'woorewards-pro')),
								array('value' => 'half_down', 'label' => __("Round to nearest integer, half way round down", 'woorewards-pro')),
							)
						)
					),
					'multiply' => array(
						'id'    => 'woorewards-pro' . '_multiply',
						'type'  => 'text',
						'title' => __("Multiply imported points by", 'woorewards-pro'),
						'extra' => array(
							'default' => '1',
							'placeholder' => '1',
						)
					),
					'default' => array(
						'id'    => 'woorewards-pro' . '_default_pool',
						'type'  => 'lacselect',
						'title' => __("Add points to that points and rewards system", 'woorewards-pro'),
						'extra' => array(
							'maxwidth' => '400px',
							'gizmo'    => true,
							'ajax'     => 'lws_woorewards_pool_list',
						)
					),
					'file' => array(
						'id'    => 'woorewards-pro' . '_import_file',
						'type'  => 'input',
						'extra' => array(
							'value' => '',
							'placeholder' => '*.json',
							'type' => 'file',
						)
					),
					'import' => array(
						'id'    => 'import-points',
						'type'  => 'custom',
						'title' => '',
						'extra' => array(
							'gizmo'   => true,
							'content' => sprintf(
								'<button type="submit" name="lws_wre_points_action" value="import" class="lws-adm-btn">%s</button>',
								__("Import", 'woorewards-pro')
							)
						)
					),
				)
			),
		);
		return $groups;
	}

	// $support = array with 'select' and 'texts'
	function addSupportText($support, $slug)
	{
		if ('woorewards' != $slug)
			return $support;

		$support['texts']['howto'] = <<<EOT
<h2>Setup Help</h2>
<p>In order to help you, we need detailed information about what you're trying to achieve.<br/>
Please provide the following information in your request</p>
<ul>
<li>How will your customers earn points ?</li>
<li>What are the rewards you want to offer ?</li>
<li>Do you plan on using sponsorship ?</li>
<li>Do you plan on using leveling points and rewards systems ?</li>
<li>How do you plan to display loyalty information to your customers ?</li>
</ul>
<h2>Your request</h2>
<p>Please provide information as detailed as possible.</p>
EOT;
		return $support;
	}

	function deleteAllData()
	{
		error_log("[WooRewards-Pro] Delete everything");

		\delete_option('lws_woorewards_pro_version');

		// delete badge posts
		$badges = \get_posts(array(
			'numberposts' => -1,
			'post_type' => 'lws_badge',
			'post_status' => array('publish', 'private', 'draft', 'pending', 'future', 'trash', 'auto-draft', 'inherit'),
			'update_post_meta_cache' => false,
			'update_post_term_cache' => false,
			'cache_results'  => false
		));
		foreach ($badges as $badge)
		{
			\wp_delete_post($badge->Id, true);
		}

		global $wpdb;

		// achievememts
		foreach (array('lws-wre-achievement') as $post_type)
		{
			foreach ($wpdb->get_col("SELECT ID FROM {$wpdb->posts} WHERE post_type='{$post_type}'") as $post_id)
				\wp_delete_post($post_id, true);
		}

		// user meta
		$wpdb->query("DELETE FROM {$wpdb->usermeta} WHERE meta_key LIKE 'lws_woorewards_%' OR meta_key LIKE 'lws_wre_event_%'");
		$wpdb->query("DELETE FROM {$wpdb->usermeta} WHERE meta_key IN ('lws-loyalty-done-steps','woorewards_special_title','woorewards_special_title_position')");

		// post meta
		$wpdb->query("DELETE FROM {$wpdb->postmeta} WHERE meta_key IN ('lws_woorewards_event_points_sponsorship','woorewards_freeproduct', 'woorewards_permanent','lws_woorewards_auto_apply','woorewards_reminder_done')");

		// mails
		foreach (array('wr_new_reward', 'wr_available_unlockables', 'wr_sponsored', 'couponreminder', 'pointsreminder') as $template)
		{
			\delete_option('lws_mail_subject_' . $template);
			\delete_option('lws_mail_preheader_' . $template);
			\delete_option('lws_mail_title_' . $template);
			\delete_option('lws_mail_header_' . $template);
			\delete_option('lws_mail_template_' . $template);
			\delete_option('lws_mail_bcc_admin_' . $template);
		}

		// clean options
		foreach (array(
			'lws_wooreward_max_sponsorship_count',
			'lws_wre_product_points_preview',
			'lws_wre_cart_points_preview',
		) as $opt)
		{
			\delete_option($opt);
		}
	}
}
