<?php
namespace LWS\WOOREWARDS\PRO\Ui;

// don't call the file directly
if( !defined( 'ABSPATH' ) ) exit();

/** Add a user birthday field. */
class BirthdayField
{
	static function register()
	{
		$registration = \get_option('lws_woorewards_registration_birthday_field');
		$detail       = \get_option('lws_woorewards_myaccount_detail_birthday_field');
		$myaccount    = \get_option('lws_woorewards_myaccount_register_birthday_field');

		if( $registration || $detail || $myaccount )
		{
			$me = new self();

			if( $registration )
			{
				\add_filter('woocommerce_checkout_fields', array($me, 'checkout'));
			}
			if( $detail )
			{
				\add_action('woocommerce_edit_account_form', array($me, 'myaccountDetailForm'));
				\add_action('woocommerce_save_account_details', array($me, 'myaccountDetailSave'));
			}
			if( $myaccount )
			{
				\add_action('woocommerce_register_form', array($me, 'myaccountRegisterForm'));
				\add_filter('woocommerce_process_registration_errors', array($me, 'myaccountRegisterValidation'), 10, 4);
				\add_action('woocommerce_created_customer', array($me, 'myaccountRegisterSave'), 10, 1);
			}

			\add_action('show_user_profile', array($me, 'showProfileBirthday'));
			\add_action('edit_user_profile', array($me, 'showProfileBirthday'));
			\add_action('personal_options_update', array($me, 'saveProfileBirthday'));
			\add_action('edit_user_profile_update', array($me, 'saveProfileBirthday'));
		}
	}

	protected function getDefaultBirthdayMetaKey()
	{
		return 'billing_birth_date';
	}

	function checkout($fields)
	{
		$fields['account'][$this->getDefaultBirthdayMetaKey()] = array(
			'type'        => 'date',
			'label'       => __("Birthday", 'woorewards-pro'),
			'required'    => false
		);
		return $fields;
	}

	function myaccountRegisterForm()
	{
		$field = $this->getDefaultBirthdayMetaKey();
		$label = __("Birthday", 'woorewards-pro');

		echo "<p class='woocommerce-form-row woocommerce-form-row--wide form-row form-row-wide'>";
		echo "<label for='{$field}'>$label</label>";
		echo "<input type='date' class='woocommerce-Input woocommerce-Input--text input-text' name='{$field}' id='{$field}' />";
		echo "</p>";
	}

	function myaccountRegisterValidation($validation_error, $username, $password, $email)
	{
		$birtday = $this->grabBirtdayFromPost();
		if( false === $birtday )
			$validation_error->add($field, __("Invalid date format for birthday", 'woorewards-pro'), 'birtday');
		return $validation_error;
	}

	function myaccountRegisterSave($userId)
	{
		$birtday = $this->grabBirtdayFromPost();
		\update_user_meta($userId, $this->getDefaultBirthdayMetaKey(), $birtday);
	}

	function myaccountDetailForm()
	{
		$userId = \get_current_user_id();
		$field = $this->getDefaultBirthdayMetaKey();
		$label = __("Birthday", 'woorewards-pro');
		$value = \esc_attr(\get_user_meta($userId, $field, true));

		echo "<p class='woocommerce-form-row woocommerce-form-row--wide form-row form-row-wide'>";
		echo "<label for='{$field}'>{$label}</label>";
		echo "<input type='date' class='woocommerce-Input woocommerce-Input--text input-text' name='{$field}' id='{$field}' value='{$value}' />";
		echo "</p><div class='clear'></div>";
	}

	function myaccountDetailSave($userId)
	{
		$birtday = $this->grabBirtdayFromPost();
		if( $birtday !== false )
			\update_user_meta($userId, $this->getDefaultBirthdayMetaKey(), $birtday);
		else
			\wc_add_notice(__("Invalid date format for birthday", 'woorewards-pro'), 'error');
	}

	function grabBirtdayFromPost()
	{
		$field = $this->getDefaultBirthdayMetaKey();
		$birtday = !empty($_POST[$field]) ? \wc_clean($_POST[$field]): '';
		if( !empty($birtday) )
		{
			if( empty(\date_create($birtday)) )
			{
				\wc_add_notice(__("Invalid date format for birthday", 'woorewards-pro'), 'error');
				$birtday = false;
			}
		}
		return $birtday;
	}

	function showProfileBirthday($user)
	{
		$field = $this->getDefaultBirthdayMetaKey();
		$label = __("Birthday", 'woorewards-pro');
		$value = \esc_attr(\get_user_meta($user->ID, $field, true));
		?>
		<table class="form-table">
			<tr>
				<th><label for='<?php echo $field; ?>'><?php echo $label; ?></label></th>
				<td>
					<input type='date' name='<?php echo $field; ?>' id='<?php echo $field; ?>' value='<?php echo $value; ?>' />
				</td>
			</tr>
		</table>
		<?php
	}

	function saveProfileBirthday($userId)
	{
		if ( !current_user_can( 'edit_user', $userId ) ) {
			return false;
		}
		$field = $this->getDefaultBirthdayMetaKey();
		update_user_meta( $userId, $field, $_POST[$field] );
	}

}
