<?php
namespace LWS\WOOREWARDS\PRO\Ui;

// don't call the file directly
if( !defined( 'ABSPATH' ) ) exit();

/** Display a message in WooCommerce new order emails. */
class NewOrderEmail
{

	static function register()
	{
		// If enabled, add the action
		if( \get_option('lws_woorewards_wc_new_order_enable'))
		{
			$me = new self();
			\add_action('woocommerce_email_order_meta', array($me,"addEmailMessage"), 10, 3);
		}
	}

	/*
	 * @param $order Order Object
	 * @param $sent_to_admin If this email is for administrator or for a customer
	 * @param $plain_text HTML or Plain text (can be configured in WooCommerce > Settings > Emails)
	 */
	function addEmailMessage($order, $sent_to_admin, $plain_text)
	{
		$status = \apply_filters('lws_woorewards_email_order_meta_status', array('processing', 'completed'));
		if( \in_array($order->get_status('edit'), $status) )
		{
			$content = \get_option('lws_woorewards_wc_new_order_content', __("With this order, you will earn [wr_wc_order_points]", 'woorewards-pro'));
			$content = \apply_filters('wpml_translate_single_string', $content, 'Widgets', "WooRewards - New Order Email Message - Earning Points");
			if( false !== strpos($content, '[wr_wc_order_points]') || false !== strpos($content, '[order_points]') || false !== strpos($content, '[points_name]') || false !== strpos($content, '[system_name]') )
			{
				$items = $this->getOrderLoyaltyInformation($order);
				if( !$items )
					return; // nothing is better than a truncated sentance

				$contents = array();
				foreach($items as $item)
				{
					$contents[] = str_replace(
						array('[wr_wc_order_points]', '[order_points]', '[points_name]', '[system_name]'),
						$item,
						$content
					);
				}
				$content = implode('<br>', $contents);
			}

			if( $plain_text )
			{
				static $replace = array(
					"<br" => "\n<br",
					"</p>" => "</p>\n\n",
					"</td>" => "</td>\t",
					"</tr>" => "</tr>\n",
					"<table" => "\n<table",
					"</thead>" => "</thead>\n",
					"</tbody>" => "</tbody>\n",
					"</table>" => "</table>\n",
				);
				$content = str_replace(array_keys($replace), array_values($replace), $content);
				$content = trim(\wp_kses($content, array()));
				$content = \wp_kses($content, array()); // filter out any html
			}
			else
			{
				$content .= "<br/>";
			}
			echo $content;
		}
	}

	/* Search for order loyalty information */
	function getOrderLoyaltyInformation($order)
	{
		$items = array();

		/* Load active and filter selected pools */
		$pools = \LWS_WooRewards_Pro::getActivePools();
		$selected = \lws_get_option('lws_woorewards_wc_new_order_pools', array());
		if( $selected && is_array($selected) )
			$pools = $pools->filter(function($p)use($selected){return in_array($p->getId(), $selected);});

		$pools = $pools->asArray();

		$userId = $order->get_customer_id('edit');
		if( $pools && $userId )
		{
			foreach( $pools as $pool )
			{
				if($pool->userCan($userId))
				{
					$poolName = $pool->getName();
					$sum = 0;

					foreach( $pool->getEvents()->asArray() as $event )
					{
						if( \is_a($event, 'LWS\WOOREWARDS\PRO\Events\I_CartPreview') )
						{
							$cat = $event->getCategories();
							if(!isset($cat['sponsorship']))
							{
								if( 0 < ($points = $event->getPointsForOrder($order)) )
								{
									$sum += $points;
								}
							}
						}
					}

					if( $sum > 0 )
					{
						$points = \LWS_WooRewards::formatPointsWithSymbol($sum, $poolName);
						$title  = $pool->getOption('display_title');
						$items[] = array(
							'wr_wc_order_points' => sprintf(_x('%1$s in %2$s', 'Order email: [X points] in [system]', 'woorewards-pro'), $points, $title),
							'order_points'       => $sum,
							'points_name'        => \LWS_WooRewards::getPointSymbol($sum, $poolName),
							'system_name'        => $title,
						);
					}
				}
			}
		}
		return $items;
	}
}
