<?php
namespace LWS\WOOREWARDS\PRO\WC;

// don't call the file directly
if( !defined( 'ABSPATH' ) ) exit();

/** New feature on WC cart
 * * Automatically add permanent coupon to the cart.
 * * Add product associated with a free product coupon. */
class Cart
{
	public function __construct()
	{
		\add_action('woocommerce_before_cart', array($this, 'completeCart'));
		if( \get_option('lws_woorewards_apply_coupon_by_reload', '') )
			\add_action('wp_loaded', array($this, 'applyCoupon'), 0);

		// act just before wc
		\add_action('wp_ajax_woocommerce_apply_coupon', array($this, 'addFreeProduct'), 0);
		\add_action('wp_ajax_nopriv_woocommerce_apply_coupon', array($this, 'addFreeProduct'), 0);
		// WC AJAX can be used for frontend ajax requests.
		\add_action('wc_ajax_apply_coupon', array($this, 'addFreeProduct'), 0);

		\add_action('woocommerce_before_cart_totals', array($this, 'freeProductPicker'));
		\add_action('woocommerce_review_order_before_order_total', array($this, 'freeProductPicker'));
		/** provide for convenience on exotic coupon/cart solutions
		 * Echo picker popup html if cart contains any freeproduct requiring it */
		\add_action('lws_woorewards_free_product_cart_picker', array($this, 'freeProductPicker'));

		\add_filter('woocommerce_cart_totals_coupon_label', array($this, 'getCouponLabel'), 1000, 2);
		\add_filter('woocommerce_removed_coupon', array($this, 'removedCoupon'), 10, 1);
		\add_filter('wcs_bypass_coupon_removal', array($this, 'wcsBypass'), 10, 2);
	}

	/** WCS does not support not recurring coupon if cart contains subscription.
	 *	Why it works anyway in most case is magic.
	 *	But here, we force him to ignore our free product coupon.
	 *	@see WC_Subscriptions_Coupon::remove_coupons() near line 581  */
	function wcsBypass($bypass, $coupon)
	{
		if( $this->isFreeProductCoupon($coupon) )
			$bypass = true;
		return $bypass;
	}

	/** add indication about the free product if any */
	function getCouponLabel($text, $coupon)
	{
		if( $this->isFreeProductCoupon($coupon) )
		{
			$code = $coupon->get_code();
			$limit = $coupon->get_limit_usage_to_x_items('edit');
			foreach (\WC()->cart->get_cart() as $cart_item)
			{
				if (isset($cart_item['product_id']) && ($productId = $cart_item['product_id'])
				&& isset($cart_item['woorewards-freeproduct']) && $cart_item['woorewards-freeproduct'] == $code)
				{
					if (!(isset($cart_item['woorewards-idle-products']) && count($cart_item['woorewards-idle-products']) > 1))
					{
						$link = sprintf("<a target='_blank' href='%s'>%s</a>", \esc_attr(\get_permalink($productId)), \get_the_title($productId));
						$text .= '<br/>' . sprintf(__("Free %s", 'woorewards-pro'), $link);
					}
					if( 0 >= --$limit )
						break;
				}
			}
		}
		return $text;
	}

	/** Do before WC actions.
	 * If the coupon is a WooRewards Free product, add that product in cart if not already in. */
	function addFreeProduct()
	{
		if( !function_exists('\wc_coupons_enabled'))
			return;
		if( !isset($_POST['coupon_code']) || empty($_POST['coupon_code']) )
			return;
		if( !\wc_coupons_enabled() )
			return;

		$code = \wc_format_coupon_code(\sanitize_text_field(\wp_unslash($_POST['coupon_code'])));
		$coupon = new \WC_Coupon($code);

		$applied_coupons = \WC()->cart->get_applied_coupons();
		foreach ( $applied_coupons as $code ) {
			$cart_coupon = new \WC_Coupon( $code );
			if( $cart_coupon->get_individual_use() && false === apply_filters('woocommerce_apply_with_individual_use_coupon', false, $coupon, $cart_coupon, $applied_coupons) )
				return;
		}

		if( $this->isFreeProductCoupon($coupon) && $coupon->get_product_ids() )
		{
			$discounts = new \LWS\WOOREWARDS\PRO\WC\Discounts(\WC()->cart);
			$omit = array(
				'min_amount' => false, // perhaps only the coming product price is missing
				'products' => false,   // product is not in cart yet
				'excluded' => false,
			);
			if( $discounts->isAllowedCoupon($coupon, $omit, true) )
			{
				$this->addTheProduct($coupon);
			}
		}
	}

	function addTheProduct($coupon)
	{
		$picker = new \LWS\WOOREWARDS\PRO\Ui\ChooseFreeProduct;
		$ids = $picker->getCouponProducts($coupon);

		$code = $coupon->get_code();
		foreach( \WC()->cart->get_cart() as $cart_item )
		{
			if( isset($cart_item['woorewards-freeproduct']) && $cart_item['woorewards-freeproduct'] == $code )
				return; // do not add twice for the same coupon
		}

		\WC()->cart->add_to_cart(reset($ids), 1, 0, array(), array(
			'unique_key' => \md5($coupon->get_code() . microtime() . rand()),
			'woorewards-freeproduct' => $code,
			'woorewards-idle-products' => $ids,
		));
	}

	/** If cart coupon requires it, append the popup over the cart */
	function freeProductPicker()
	{
		foreach (\WC()->cart->get_cart() as $cart_item)
		{
			if (isset($cart_item['product_id']) && ($productId = $cart_item['product_id'])
			&& isset($cart_item['woorewards-freeproduct']))
			{
				if (isset($cart_item['woorewards-idle-products']) && count($cart_item['woorewards-idle-products']) > 1)
				{
					if (\WC()->cart->has_discount($cart_item['woorewards-freeproduct']))
					{
						$coupon = new \WC_Coupon($cart_item['woorewards-freeproduct']);
						if ($coupon){
							$picker = new \LWS\WOOREWARDS\PRO\Ui\ChooseFreeProduct;
							echo $picker->getFreeProductPopup('', $coupon);
						}
					}
				}
			}
		}
	}

	/* If a coupon is removed and it's a free product, also remove the product */
	function removedCoupon($code)
	{
		$coupon = new \WC_Coupon($code);
		$cart = \WC()->instance()->cart;
		if( \get_post_meta($coupon->get_id(), 'woorewards_freeproduct', true) == 'yes' )
		{
			foreach( \WC()->cart->get_cart() as $cart_item )
			{
				if( isset($cart_item['woorewards-freeproduct']) && $cart_item['woorewards-freeproduct'] == $code )
				{
					$cart->remove_cart_item($cart_item['key']);
				}
			}
		}

		if( $this->isAutoApplyCoupon($coupon) )
			\update_post_meta($coupon->get_id(), 'lws_woorewards_auto_apply', 'idle');
	}

	protected function isFreeProductCoupon($coupon)
	{
		if( $coupon->get_discount_type() != 'percent' )
			return false;
		if( !$coupon->get_product_ids() )
			return false;
		if( \get_post_meta($coupon->get_id(), 'woorewards_freeproduct', true) != 'yes' )
			return false;
		return true;
	}

	protected function isAutoApplyCoupon($coupon)
	{
		return ('on' == \get_post_meta($coupon->get_id(), 'lws_woorewards_auto_apply', true));
	}

	protected function isVariableDiscountCoupon($coupon)
	{
		if( $coupon->get_discount_type() == 'percent' )
			return false;
		if( \get_post_meta($coupon->get_id(), 'reward_origin', true) != 'lws_woorewards_pro_unlockables_variablediscount' )
			return false;
		return true;
	}

	/** if $_REQUEST contains a coupon to add, we apply it on the cart */
	public function applyCoupon()
	{
		if( !function_exists('\wc_coupons_enabled'))
			return;
		if( !\wc_coupons_enabled() )
			return;

		if( isset($_REQUEST['wrac_n']) && isset($_REQUEST['wrac_c']) && \wp_verify_nonce($_REQUEST['wrac_n'], 'wr_apply_coupon') )
		{
			if( !empty($code = \sanitize_text_field($_REQUEST['wrac_c'])) )
			{
				$stack = array_keys(\WC()->cart->get_coupons());
				if( !in_array($code, $stack) )
				{
					if( $id = \wc_get_coupon_id_by_code($code) )
					{
						$wcCoupon = new \WC_Coupon($id);
						$freeProduct = ($this->isFreeProductCoupon($wcCoupon) && $wcCoupon->get_product_ids());
						$error = false;

						if( $freeProduct )
						{
							$discounts = new \LWS\WOOREWARDS\PRO\WC\Discounts(\WC()->cart);
							$omit = array(
								'min_amount' => false, // perhaps only the coming product price is missing
								'products' => false,   // product is not in cart yet
								'excluded' => false,
							);
							if( $discounts->isAllowedCoupon($wcCoupon, $omit, true) )
							{
								$this->addTheProduct($wcCoupon);
							}
							else if( $error = $discounts->getLastError() )
							{
								\wc_add_notice($error, 'error');
							}
						}

						if( !$error )
							\WC()->cart->apply_coupon($code);
					}

					$redirect = \remove_query_arg(array('wrac_n', 'wrac_c'));
					if( $redirect = \apply_filters('lws_woorewards_redirect_url_after_coupon_apply', $redirect) )
					{
						\wp_redirect($redirect);
						exit();
					}
				}
			}
		}
	}

	/** Automatically add any current user available auto_apply coupon to the cart */
	public function completeCart()
	{
		$user = \wp_get_current_user();
		if( !empty($user->ID) )
		{
			$stack = array_keys(\WC()->cart->get_coupons());
			foreach($this->loadCoupons($user) as $coupon)
			{
				if( !in_array($coupon->title, $stack) )
				{
					$wcCoupon = new \WC_Coupon($coupon->ID);
					$freeProduct = ($this->isFreeProductCoupon($wcCoupon) && $wcCoupon->get_product_ids());
					$discounts = new \LWS\WOOREWARDS\PRO\WC\Discounts(\WC()->cart);
					$error = false;

					if( $freeProduct )
					{
						$omit = array(
							'min_amount' => false, // perhaps only the coming product price is missing
							'products' => false,   // product is not in cart yet
							'excluded' => false,
						);
						if( $discounts->isAllowedCoupon($wcCoupon, $omit, true) )
						{
							$this->addTheProduct($wcCoupon);
							// redo discount class with new cart
							$discounts = new \LWS\WOOREWARDS\PRO\WC\Discounts(\WC()->cart);
						}
						else
							$error = true;
					}

					if( !$error && $discounts->isAllowedCoupon($wcCoupon, true) )
					{
						$limited = \max($wcCoupon->get_usage_limit('edit'), $wcCoupon->get_usage_limit_per_user('edit'));
						if( $limited && \get_post_meta($coupon->ID, 'lws_woorewards_auto_apply', true) )
								\update_post_meta($coupon->ID, 'lws_woorewards_auto_apply', 'off');

						\WC()->cart->apply_coupon($coupon->title);
					}
				}
			}
		}
	}

	/** Usable auto-applied coupons.
	 *  return array of post{ID, title} */
	private function loadCoupons($user)
	{
		if( empty($user->user_email) )
			return array();
		$todayDate = strtotime(date('Y-m-d'));

		global $wpdb;
		$sql = <<<EOT
SELECT p.ID as ID, p.post_title as title from {$wpdb->posts} as p
INNER JOIN {$wpdb->postmeta} as mail ON p.ID = mail.post_id AND mail.meta_key='customer_email'
INNER JOIN {$wpdb->postmeta} as w ON p.ID = w.post_id AND w.meta_key='lws_woorewards_auto_apply' AND w.meta_value='on'
LEFT JOIN {$wpdb->postmeta} as l ON p.ID = l.post_id AND l.meta_key='usage_limit'
LEFT JOIN {$wpdb->postmeta} as u ON p.ID = u.post_id AND u.meta_key='usage_count'
LEFT JOIN {$wpdb->postmeta} as e ON p.ID = e.post_id AND e.meta_key='expiry_date'
WHERE mail.meta_value=%s AND post_type = 'shop_coupon' AND post_status = 'publish'
AND (e.meta_value IS NULL OR e.meta_value = '' OR e.meta_value >= '{$todayDate}')
AND (u.meta_value < l.meta_value OR u.meta_value IS NULL OR l.meta_value IS NULL OR l.meta_value=0)
EOT;
		return $wpdb->get_results($wpdb->prepare($sql, serialize(array($user->user_email))));
	}
}
