<?php
namespace LWS\WOOREWARDS\PRO\WC;

// don't call the file directly
if( !defined( 'ABSPATH' ) ) exit();

/** Provided for convenience to test coupon validity.
 *  Since all usefull methods from WC are protected. */
class Discounts extends \WC_Discounts
{
	/** @param WC_Cart|WC_Order $object Cart or order object.  */
	public function __construct( $object = null )
	{
		parent::__construct($object);
		$this->lastError = false;
	}

	/** Silent test.
	 *	@param $coupon (WC_Coupon) Coupon data.
	 *	@param $test (true|array) if array, set key => true or key => false, where key is the test:
	 *  *  exists
	 *  *  usage
	 *  *  user_usage
	 *  *  expiry
	 *  *  min_amount
	 *  *  max_amount
	 *  *  products
	 *  *  categories
	 *  *  excluded
	 *  *  eligible
	 *  *  external (use filter)
	 *	@param $omitted (bool) if a test is omitted in array above, use that as default value.
	 *	@return bool */
	public function isAllowedCoupon($coupon, $test=true, $omitted=true)
	{
		$this->lastError = false;
		try{
			if( true === $test )
			{
				return (true === $this->is_coupon_valid($coupon));
			}
			else
			{
				if( !is_array($test) )
					$test = array();
				$test = array_merge(
					array_fill_keys(
						array('exists','usage','user_usage','expiry','min_amount','max_amount','products','categories','excluded','eligible','external'),
						$omitted
					),
					$test
				);

				if( $test['exists'] )     $this->validate_coupon_exists($coupon);
				if( $test['usage'] )      $this->validate_coupon_usage_limit($coupon);
				if( $test['user_usage'] ) $this->validate_coupon_user_usage_limit($coupon);
				if( $test['expiry'] )     $this->validate_coupon_expiry_date($coupon);
				if( $test['min_amount'] ) $this->validate_coupon_minimum_amount($coupon);
				if( $test['max_amount'] ) $this->validate_coupon_maximum_amount($coupon);
				if( $test['products'] )   $this->validate_coupon_product_ids($coupon);
				if( $test['categories'] ) $this->validate_coupon_product_categories($coupon);
				if( $test['excluded'] )   $this->validate_coupon_excluded_items($coupon);
				if( $test['eligible'] )   $this->validate_coupon_eligible_items($coupon);

				if( $test['external'] )
				{
					if( !\apply_filters('woocommerce_coupon_is_valid', true, $coupon, $this) )
						return false;
				}
			}
		}
		catch( \Exception $e )
		{
			$this->lastError = \apply_filters(
				'woocommerce_coupon_error',
				is_numeric($e->getMessage()) ? $coupon->get_coupon_error($e->getMessage()) : $e->getMessage(),
				$e->getCode(),
				$coupon
			);
			return false;
		}
		return true;
	}

	function getLastError()
	{
		return $this->lastError;
	}
}
