<?php
namespace LWS\WOOREWARDS\PRO\WC;

// don't call the file directly
if( !defined( 'ABSPATH' ) ) exit();

/** Refund points at order refund. */
class Order
{
	static function install()
	{
		$me = new self();
		$orderStatuses = \get_option('lws_woorewards_refund_on_status');
		if( $orderStatuses && is_array($orderStatuses) )
		{
			foreach( array_unique($orderStatuses) as $status )
				\add_action('woocommerce_order_status_'.$status, array($me, 'refund'), 998, 2); // priority late to let someone change amount and wc to save order
		}
	}

	function refund($orderId, $order)
	{
		if( !\get_post_meta($orderId, 'lws_woorewards_points_refunded', true) )
		{
			if( $order )
			{
				$logs = \LWS\WOOREWARDS\Core\PointStack::queryTrace(array(
					'order_id' => $orderId,
					'blog_id'  => \get_current_blog_id(),
				));
				if( $logs )
				{
					$sort = array();
					foreach( $logs as $log )
					{
						if( isset($sort[$log->stack][$log->user_id]) )
							$sort[$log->stack][$log->user_id] += $log->move;
						else
							$sort[$log->stack][$log->user_id] = $log->move;
					}
					\update_post_meta($orderId, 'lws_woorewards_points_refunded', $sort);

					$reason = \LWS\WOOREWARDS\Core\trace::byOrder($order)->setOrigin('refund')
						->setReason(array('Refund Order #%s', $order->get_order_number()), 'woorewards-pro');

					foreach( $sort as $stack => $value )
					{
						foreach( $value as $userId => $points )
						{
							if( $points != 0 )
							{
								$manager = new \LWS\WOOREWARDS\Core\PointStack($stack, $userId);
								$manager->sub($points, $reason);

								$order->add_order_note(sprintf(
									__('<b>%1$s</b> removed in <i>%4$s</i> from customer <i>[%2$d]</i> since order passed to <i>%3$s</i>.', 'woorewards-pro'),
									\LWS_WooRewards::formatPointsWithSymbol($points, ''),
									$userId,
									$order->get_status(),
									$stack
								));
							}
						}
					}
				}
			}
		}
	}

	/** Never call, only to have poedit/wpml able to extract the sentance. */
	private function poeditDeclare()
	{
		__("Refund Order #%s", 'woorewards-pro');
	}
}
